@@ -2,8 +2,8 @@
 Builds bootable CD images that have PXE-equivalent behavior
 for all Cobbler distros/profiles/systems currently in memory.
 
-Copyright 2006-2009, Red Hat, Inc
-Michael DeHaan <mdehaan@redhat.com>
+Copyright 2006-2009, Red Hat, Inc and Others
+Michael DeHaan <michael.dehaan AT gmail>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -82,6 +82,9 @@ class BuildIso:
 
 
     def make_shorter(self,distname):
+        """
+        Return a short distro identifier
+        """
         if self.distmap.has_key(distname):
             return self.distmap[distname]
         else:
@@ -151,7 +154,7 @@ class BuildIso:
              cfg.write("  MENU LABEL %s\n" % profile.name)
              cfg.write("  kernel %s.krn\n" % distname)
 
-             data = utils.blender(self.api, True, profile)
+             data = utils.blender(self.api, False, profile)
              if data["kickstart"].startswith("/"):
                  data["kickstart"] = "http://%s:%s/cblr/svc/op/ks/profile/%s" % (
                      data["server"], self.api.settings().http_port, profile.name
@@ -161,14 +164,29 @@ class BuildIso:
              if dist.breed == "suse":
                  if data["proxy"] != "":
                      append_line += " proxy=%s" % data["proxy"]
-                 append_line += " autoyast=%s" % data["kickstart"]
+                 if data["kernel_options"].has_key("install") and data["kernel_options"]["install"] != "":
+                     append_line += " install=%s" % data["kernel_options"]["install"]
+                     del data["kernel_options"]["install"]
+                 else:
+                     append_line += " install=http://%s:%s/cblr/links/%s" % (
+                         data["server"], self.api.settings().http_port, dist.name
+                     )
+                 if data["kernel_options"].has_key("autoyast") and data["kernel_options"]["autoyast"] != "":
+                     append_line += " autoyast=%s" % data["kernel_options"]["autoyast"]
+                     del data["kernel_options"]["autoyast"]
+                 else:
+                     append_line += " autoyast=%s" % data["kickstart"]
+
              if dist.breed == "redhat":
+                 if data["proxy"] != "":
+                    append_line += " proxy=%s http_proxy=%s" % (data["proxy"], data["proxy"])
                  append_line += " ks=%s" % data["kickstart"]
+
              if dist.breed in ["ubuntu","debian"]:
                  append_line += " auto-install/enable=true url=%s" % data["kickstart"]
                  if data["proxy"] != "":
                      append_line += " mirror/http/proxy=%s" % data["proxy"]
-             append_line = append_line + " %s\n" % data["kernel_options"]
+             append_line += self.add_remaining_kopts(data["kernel_options"])
              cfg.write(append_line)
 
              length=len(append_line)
@@ -191,23 +209,38 @@ class BuildIso:
              cfg.write("  MENU LABEL %s\n" % system.name)
              cfg.write("  kernel %s.krn\n" % distname)
 
-             bdata = utils.blender(self.api, True, system)
-             if bdata["kickstart"].startswith("/"):
-                 bdata["kickstart"] = "http://%s:%s/cblr/svc/op/ks/system/%s" % (
-                     bdata["server"], self.api.settings().http_port, system.name
+             data = utils.blender(self.api, False, system)
+             if data["kickstart"].startswith("/"):
+                 data["kickstart"] = "http://%s:%s/cblr/svc/op/ks/system/%s" % (
+                     data["server"], self.api.settings().http_port, system.name
                  )
 
              append_line = " append initrd=%s.img" % distname
              if dist.breed == "suse":
-                if bdata["proxy"] != "":
-                    append_line += " proxy=%s" % bdata["proxy"]
-                append_line += " autoyast=%s" % bdata["kickstart"]
+                if data["proxy"] != "":
+                    append_line += " proxy=%s" % data["proxy"]
+                if data["kernel_options"].has_key("install") and data["kernel_options"]["install"] != "":
+                    append_line += " install=%s" % data["kernel_options"]["install"]
+                    del data["kernel_options"]["install"]
+                else:
+                    append_line += " install=http://%s:%s/cblr/links/%s" % (
+                        data["server"], self.api.settings().http_port, dist.name
+                    )
+                if data["kernel_options"].has_key("autoyast") and data["kernel_options"]["autoyast"] != "":
+                    append_line += " autoyast=%s" % data["kernel_options"]["autoyast"]
+                    del data["kernel_options"]["autoyast"]
+                else:
+                    append_line += " autoyast=%s" % data["kickstart"]
+
              if dist.breed == "redhat":
-                append_line += " ks=%s" % bdata["kickstart"]
+                if data["proxy"] != "":
+                    append_line += " proxy=%s http_proxy=%s" % (data["proxy"], data["proxy"])
+                append_line += " ks=%s" % data["kickstart"]
+
              if dist.breed in ["ubuntu","debian"]:
-                append_line += " auto-install/enable=true url=%s netcfg/disable_dhcp=true" % bdata["kickstart"]
-                if bdata["proxy"] != "":
-                    append_line += " mirror/http/proxy=%s" % bdata["proxy"]
+                append_line += " auto-install/enable=true url=%s netcfg/disable_dhcp=true" % data["kickstart"]
+                if data["proxy"] != "":
+                    append_line += " mirror/http/proxy=%s" % data["proxy"]
                 # hostname is required as a parameter, the one in the preseed is not respected
                 my_domain = "local.lan"
                 if system.hostname != "":
@@ -230,7 +263,6 @@ class BuildIso:
 
              # try to add static ip boot options to avoid DHCP (interface/ip/netmask/gw/dns)
              # check for overrides first and clear them from kernel_options
-             data = utils.blender(self.api, False, system) # don't collapse!
              my_int = None;  my_ip = None; my_mask = None; my_gw = None; my_dns = None
              if dist.breed in ["suse", "redhat"]:
                 if data["kernel_options"].has_key("netmask") and data["kernel_options"]["netmask"] != "":
@@ -243,6 +275,8 @@ class BuildIso:
              if dist.breed == "redhat":
                 if data["kernel_options"].has_key("ksdevice") and data["kernel_options"]["ksdevice"] != "":
                    my_int = data["kernel_options"]["ksdevice"]
+                   if my_int == "bootif":
+                      my_int = None
                    del data["kernel_options"]["ksdevice"]
                 if data["kernel_options"].has_key("ip") and data["kernel_options"]["ip"] != "":
                    my_ip = data["kernel_options"]["ip"]
@@ -288,7 +322,7 @@ class BuildIso:
                       if idata["management"] == True and idata["interface_type"] in ["master","bond","bridge"]:
                          # bonded/bridged management interface
                          mgmt_ints_multi.append(iname)
-                      if idata["management"] == True and idata["interface_type"] not in ["master","bond","bridge","slave","bond_slave","bridge_slave"]:
+                      if idata["management"] == True and idata["interface_type"] not in ["master","bond","bridge","slave","bond_slave","bridge_slave","bonded_bridge_slave"]:
                          # single management interface
                          mgmt_ints.append(iname)
 
@@ -296,7 +330,7 @@ class BuildIso:
                    # bonded/bridged management interface, find a slave interface
                    # if eth0 is a slave use that (it's what people expect)
                    for (iname, idata) in data["interfaces"].iteritems():
-                      if idata["interface_type"] in ["slave","bond_slave","bridge_slave"] and idata["interface_master"] == mgmt_ints_multi[0]:
+                      if idata["interface_type"] in ["slave","bond_slave","bridge_slave","bonded_bridge_slave"] and idata["interface_master"] == mgmt_ints_multi[0]:
                          slave_ints.append(iname)
 
                    if "eth0" in slave_ints:
@@ -440,7 +474,7 @@ class BuildIso:
             if distro.breed == "redhat":
                append_line += " ks=cdrom:/isolinux/%s.cfg" % descendant.name
             if distro.breed == "suse":
-               append_line += " autoyast=file:///isolinux/%s.cfg install=file:///" % descendant.name
+               append_line += " autoyast=file:///isolinux/%s.cfg install=cdrom:///" % descendant.name
                if data["kernel_options"].has_key("install"):
                   del data["kernel_options"]["install"]
             if distro.breed in ["ubuntu","debian"]:
@@ -455,8 +489,9 @@ class BuildIso:
             elif descendant.COLLECTION_TYPE == 'system':
                 kickstart_data = self.api.kickgen.generate_kickstart_for_system(descendant.name)
 
-            cdregex = re.compile("url .*\n", re.IGNORECASE)
-            kickstart_data = cdregex.sub("cdrom\n", kickstart_data)
+            if distro.breed == "redhat":
+                cdregex = re.compile("url .*\n", re.IGNORECASE)
+                kickstart_data = cdregex.sub("cdrom\n", kickstart_data)
 
             ks_name = os.path.join(isolinuxdir, "%s.cfg" % descendant.name)
             ks_file = open(ks_name, "w+")
@@ -471,7 +506,7 @@ class BuildIso:
         return
 
 
-    def run(self,iso=None,buildisodir=None,profiles=None,systems=None,distro=None,standalone=None,source=None,exclude_dns=None):
+    def run(self,iso=None,buildisodir=None,profiles=None,systems=None,distro=None,standalone=None,source=None,exclude_dns=None,mkisofs_opts=None):
 
         # the distro option is for stand-alone builds only
         if not standalone and distro is not None:
@@ -545,8 +580,13 @@ class BuildIso:
         else:
             self.generate_netboot_iso(imagesdir,isolinuxdir,profiles,systems,exclude_dns)
 
+        if mkisofs_opts == None:
+            mkisofs_opts = ""
+        else:
+            mkisofs_opts = mkisofs_opts.strip()
+
         # removed --quiet
-        cmd = "mkisofs -o %s -r -b isolinux/isolinux.bin -c isolinux/boot.cat" % iso
+        cmd = "mkisofs -o %s %s -r -b isolinux/isolinux.bin -c isolinux/boot.cat" % (iso,mkisofs_opts)
         cmd = cmd + " -no-emul-boot -boot-load-size 4"
         cmd = cmd + " -boot-info-table -V Cobbler\ Install -R -J -T %s" % buildisodir
 
