@@ -2,8 +2,8 @@
 Validates whether the system is reasonably well configured for
 serving up content.  This is the code behind 'cobbler check'.
 
-Copyright 2006-2009, Red Hat, Inc
-Michael DeHaan <mdehaan@redhat.com>
+Copyright 2006-2009, Red Hat, Inc and Others
+Michael DeHaan <michael.dehaan AT gmail>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -84,6 +84,7 @@ class BootCheck:
        self.check_service(status, "cobblerd")
     
        self.check_bootloaders(status)
+       self.check_for_wget_curl(status)
        self.check_rsync_conf(status)
        self.check_httpd(status)
        self.check_iptables(status)
@@ -143,7 +144,7 @@ class BootCheck:
        if os.path.exists("/etc/rc.d/init.d/iptables"):
            rc = utils.subprocess_call(self.logger,"/sbin/service iptables status >/dev/null 2>/dev/null", shell=True)
            if rc == 0:
-              status.append(_("since iptables may be running, ensure 69, 80, and %(xmlrpc)s are unblocked") % { "xmlrpc" : self.settings.xmlrpc_port })
+              status.append(_("since iptables may be running, ensure 69, 80/443, and %(xmlrpc)s are unblocked") % { "xmlrpc" : self.settings.xmlrpc_port })
 
    def check_yum(self,status):
        if self.checked_dist in ["debian", "ubuntu"]:
@@ -170,9 +171,9 @@ class BootCheck:
           re_arches = re.compile(r'@arches=')
           for line in f.readlines():
              if re_dists.search(line) and not line.strip().startswith("#"):
-                 status.append(_("comment 'dists' on /etc/debmirror.conf for proper debian support"))
+                 status.append(_("comment out 'dists' on /etc/debmirror.conf for proper debian support"))
              if re_arches.search(line) and not line.strip().startswith("#"):
-                 status.append(_("comment 'arches' on /etc/debmirror.conf for proper debian support"))
+                 status.append(_("comment out 'arches' on /etc/debmirror.conf for proper debian support"))
        
 
    def check_name(self,status):
@@ -197,52 +198,7 @@ class BootCheck:
 
        enabled = self.config.api.is_selinux_enabled()
        if enabled:
-           data2 = utils.subprocess_get(self.logger,"/usr/sbin/getsebool -a",shell=True)
-           for line in data2.split("\n"):
-              if line.find("httpd_can_network_connect ") != -1:
-                  if line.find("off") != -1:
-                      status.append(_("Must enable a selinux boolean to enable vital web services components, run: setsebool -P httpd_can_network_connect true"))
-              if line.find("rsync_disable_trans ") != -1:
-                  if line.find("on") != -1:
-                      status.append(_("Must enable the cobbler import and replicate commands, run: setsebool -P rsync_disable_trans=1"))
-
-           data3 = utils.subprocess_get(self.logger,"/usr/sbin/semanage fcontext -l | grep public_content_t",shell=True)
-
-           rule1 = False
-           rule2 = False
-           rule3 = False
-           selinux_msg = "/usr/sbin/semanage fcontext -a -t public_content_t \"%s\""
-           for line in data3.split("\n"):
-               if line.startswith("/tftpboot/.*"):
-                   rule1 = True
-               if line.startswith("/var/lib/tftpboot/.*"):
-                   rule2 = True
-               if line.startswith(self.settings.webdir+"/images/.*"):
-                   rule3 = True
-
-           rules = []
-           if os.path.exists("/tftpboot") and not rule1:
-               rules.append(selinux_msg % "/tftpboot/.*")
-           else:
-               if not rule2:
-                   rules.append(selinux_msg % "/var/lib/tftpboot/.*")
-           if not rule3:
-               rules.append(selinux_msg % self.settings.webdir+"/images/.*")
-           if len(rules) > 0:
-               status.append("you need to set some SELinux content rules to ensure cobbler serves content correctly in your SELinux environment, run the following: %s" % " && ".join(rules))
-
-           # now check to see that the Django sessions path is accessible
-           # by Apache
-           
-           data4 = utils.subprocess_get(self.logger,"/usr/sbin/semanage fcontext -l | grep httpd_sys_content_rw_t",shell=True)
-           selinux_msg = "you need to set some SELinux rules if you want to use cobbler-web (an optional package), run the following: /usr/sbin/semanage fcontext -a -t httpd_sys_content_rw_t \"%s\""
-           rule4 = False
-           for line in data4.split("\n"):
-               if line.startswith("/var/lib/cobbler/webui_sessions/.*"):
-                   rule4 = True
-           if not rule4:
-               status.append(selinux_msg % "/var/lib/cobbler/webui_sessions/.*")
-
+           status.append(_("SELinux is enabled. Please review the following wiki page for details on ensuring cobbler works correctly in your SELinux environment:\n    https://github.com/cobbler/cobbler/wiki/Selinux"))
 
    def check_for_default_password(self,status):
        default_pass = self.settings.default_password_crypted
@@ -313,6 +269,15 @@ class BootCheck:
        if rc.find("BIND") == -1:
            status.append("named is not installed and/or in path")
        
+   def check_for_wget_curl(self,status):
+       """
+       Check to make sure wget or curl is installed
+       """
+       rc1 = utils.subprocess_call(self.logger,"which wget")
+       rc2 = utils.subprocess_call(self.logger,"which curl")
+       if rc1 != 0 and rc2 != 0:
+           status.append("Neither wget nor curl are installed and/or available in $PATH. Cobbler requires that one of these utilities be installed.")
+
    def check_bootloaders(self,status):
        """
        Check if network bootloaders are installed
