@@ -2,8 +2,8 @@
 Builds out and synchronizes yum repo mirrors.
 Initial support for rsync, perhaps reposync coming later.
 
-Copyright 2006-2007, Red Hat, Inc
-Michael DeHaan <mdehaan@redhat.com>
+Copyright 2006-2007, Red Hat, Inc and Others
+Michael DeHaan <michael.dehaan AT gmail>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -87,14 +87,6 @@ class RepoSync:
 
         report_failure = False
         for repo in self.repos:
-
-            env = repo.environment
-
-            for k in env.keys():
-                self.logger.info("environment: %s=%s" % (k,env[k]))
-                if env[k] is not None:
-                    os.putenv(k,env[k])
-
             if name is not None and repo.name != name:
                 # invoked to sync only a specific repo, this is not the one
                 continue
@@ -109,7 +101,21 @@ class RepoSync:
 
             if not os.path.isdir(repo_path) and not repo.mirror.lower().startswith("rhn://"):
                 os.makedirs(repo_path)
-            
+
+            # set the environment keys specified for this repo, 
+            # save the old ones if they modify an existing variable
+
+            env = repo.environment
+            old_env = {}
+
+            for k in env.keys():
+                self.logger.debug("setting repo environment: %s=%s" % (k,env[k]))
+                if env[k] is not None:
+                    if os.getenv(k):
+                        old_env[k] = os.getenv(k)
+                    else:
+                        os.environ[k] = env[k]
+
             # which may actually NOT reposync if the repo is set to not mirror locally
             # but that's a technicality
 
@@ -118,10 +124,23 @@ class RepoSync:
                 try:
                     self.sync(repo) 
                     success = True
+                    break
                 except:
                     utils.log_exc(self.logger)
                     self.logger.warning("reposync failed, tries left: %s" % (x-2))
 
+            # cleanup/restore any environment variables that were 
+            # added or changed above
+
+            for k in env.keys():
+                if env[k] is not None:
+                    if old_env.has_key(k):
+                        self.logger.debug("resetting repo environment: %s=%s" % (k,old_env[k]))
+                        os.environ[k] = old_env[k]
+                    else:
+                        self.logger.debug("removing repo environment: %s=%s" % (k,env[k]))
+                        del os.environ[k]
+
             if not success:
                 report_failure = True
                 if not self.nofail:
@@ -221,7 +240,7 @@ class RepoSync:
             repo.mirror = "%s/" % repo.mirror
 
         # FIXME: wrapper for subprocess that logs to logger
-        cmd = "rsync -rltDv %s --delete --exclude-from=/etc/cobbler/rsync.exclude %s %s" % (spacer, repo.mirror, dest_path)
+        cmd = "rsync -rltDv --copy-unsafe-links --delete-after %s --delete --exclude-from=/etc/cobbler/rsync.exclude %s %s" % (spacer, repo.mirror, dest_path)
         rc = utils.subprocess_call(self.logger, cmd)
 
         if rc !=0:
@@ -446,7 +465,11 @@ class RepoSync:
         dest_path = os.path.join("/var/www/cobbler/repo_mirror", repo.name)
          
         if repo.mirror_locally:
-            mirror = repo.mirror.replace("@@suite@@",repo.os_version)
+            # NOTE: Dropping @@suite@@ replace as it is also dropped from
+            # from manage_import_debian_ubuntu.py due that repo has no os_version
+            # attribute. If it is added again it will break the Web UI!
+            #mirror = repo.mirror.replace("@@suite@@",repo.os_version)
+            mirror = repo.mirror
 
             idx = mirror.find("://")
             method = mirror[:idx]
@@ -454,17 +477,14 @@ class RepoSync:
 
             idx = mirror.find("/")
             host = mirror[:idx]
-            mirror = mirror[idx+1:]
+            mirror = mirror[idx:]
 
-            idx = mirror.rfind("/dists/")
-            suite = mirror[idx+7:]
-            mirror = mirror[:idx]
+            dists = ",".join(repo.apt_dists)
+            components = ",".join(repo.apt_components)
 
-            mirror_data = "--method=%s --host=%s --root=%s --dist=%s " % ( method , host , mirror , suite )
+            mirror_data = "--method=%s --host=%s --root=%s --dist=%s --section=%s" % (method,host,mirror,dists,components)
 
-            # FIXME : flags should come from repo instead of being hardcoded
-
-            rflags = "--passive --nocleanup"
+            rflags = "--nocleanup"
             for x in repo.yumopts:
                 if repo.yumopts[x]:
                     rflags += " %s %s" % ( x , repo.yumopts[x] ) 
@@ -481,6 +501,11 @@ class RepoSync:
                    arch = "amd64" # FIX potential arch errors
                 cmd = "%s --nosource -a %s" % (cmd, arch)
                     
+            # Set's an environment variable for subprocess, otherwise debmirror will fail
+            # as it needs this variable to exist.
+            # FIXME: might this break anything? So far it doesn't
+            os.putenv("HOME", "/var/lib/cobbler")
+
             rc = utils.subprocess_call(self.logger, cmd)
             if rc !=0:
                 utils.die(self.logger,"cobbler reposync failed")
@@ -515,7 +540,7 @@ class RepoSync:
         optgpgcheck = False
         if output:
             if repo.mirror_locally:
-                line = "baseurl=http://${server}/cobbler/repo_mirror/%s\n" % (repo.name)
+                line = "baseurl=http://${http_server}/cobbler/repo_mirror/%s\n" % (repo.name)
             else:
                 mstr = repo.mirror
                 if mstr.startswith("/"):
