@@ -2,8 +2,8 @@
 Builds out filesystem trees/data based on the object tree.
 This is the code behind 'cobbler sync'.
 
-Copyright 2006-2009, Red Hat, Inc
-Michael DeHaan <mdehaan@redhat.com>
+Copyright 2006-2009, Red Hat, Inc and Others
+Michael DeHaan <michael.dehaan AT gmail>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -42,6 +42,7 @@ import item_system
 from Cheetah.Template import Template
 import clogger
 from utils import _
+import cobbler.module_loader as module_loader
 
 
 class BootSync:
@@ -121,6 +122,7 @@ class BootSync:
                 self.logger.info("copying files for distro: %s" % d.name)
                 self.pxegen.copy_single_distro_files(d,
                                                      self.settings.webdir,True)
+                self.pxegen.write_templates(d,write_file=True)
             except CX, e:
                 self.logger.error(e.value)
 
@@ -128,9 +130,7 @@ class BootSync:
         self.pxegen.make_pxe_menu()
 
         if self.settings.manage_dhcp:
-            self.logger.info("rendering DHCP files")
-            self.dhcp.write_dhcp_file()
-            self.dhcp.regen_ethers()
+            self.write_dhcp()
         if self.settings.manage_dns:
             self.logger.info("rendering DNS files")
             self.dns.regen_hosts()
@@ -140,6 +140,8 @@ class BootSync:
            # xinetd.d/tftpd, basically
            self.logger.info("rendering TFTPD files")
            self.tftpd.write_tftpd_files()
+           # copy in boot_files
+           self.tftpd.write_boot_files()
 
         self.logger.info("cleaning link caches")
         self.clean_link_cache()
@@ -212,6 +214,37 @@ class BootSync:
         utils.rmtree_contents(self.yaboot_cfg_dir,logger=self.logger)
         utils.rmtree_contents(self.rendered_dir,logger=self.logger)
 
+    def write_dhcp(self):
+        self.logger.info("rendering DHCP files")
+        self.dhcp.write_dhcp_file()
+        self.dhcp.regen_ethers()
+
+    def sync_dhcp(self):
+        restart_dhcp = str(self.settings.restart_dhcp).lower()
+        which_dhcp_module = module_loader.get_module_from_file("dhcp","module",just_name=True).strip()
+
+        if self.settings.manage_dhcp:
+            self.write_dhcp()
+            if which_dhcp_module == "manage_isc":
+                service_name = utils.dhcp_service_name(self.api)
+                if restart_dhcp != "0":
+                    rc = utils.subprocess_call(self.logger, "dhcpd -t -q", shell=True)
+                    if rc != 0:
+                       self.logger.error("dhcpd -t failed")
+                       return False
+                    service_restart = "service %s restart" % service_name
+                    rc = utils.subprocess_call(self.logger, service_restart, shell=True)
+                    if rc != 0:
+                       self.logger.error("%s failed" % service_name)
+                       return False
+            elif which_dhcp_module == "manage_dnsmasq":
+                if restart_dhcp != "0":
+                    rc = utils.subprocess_call(self.logger, "service dnsmasq restart")
+                    if rc != 0:
+                       self.logger.error("service dnsmasq restart failed")
+                       return False
+        return True
+
     def clean_link_cache(self):
         for dirtree in [os.path.join(self.bootloc,'images'), self.settings.webdir]:
             cachedir = os.path.join(dirtree,'.link_cache')
