@@ -3,8 +3,8 @@ python API module for Cobbler
 see source for cobbler.py, or pydoc, for example usage.
 CLI apps and daemons should import api.py, and no other cobbler code.
 
-Copyright 2006-2009, Red Hat, Inc
-Michael DeHaan <mdehaan@redhat.com>
+Copyright 2006-2009, Red Hat, Inc and Others
+Michael DeHaan <michael.dehaan AT gmail>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -49,11 +49,14 @@ from utils import _
 import logging
 import time
 import random
+import simplejson
 import os
 import xmlrpclib
 import traceback
 import exceptions
 import clogger
+import tempfile
+import urllib2
 
 import item_distro
 import item_profile
@@ -69,7 +72,7 @@ INFO  = 10
 DEBUG = 5
 
 # FIXME: add --quiet depending on if not --verbose?
-RSYNC_CMD =  "rsync -a %s '%s' %s --exclude-from=/etc/cobbler/rsync.exclude --progress"
+RSYNC_CMD =  "rsync -a %s '%s' %s --progress"
 
 # notes on locking:
 # BootAPI is a singleton object
@@ -116,7 +119,7 @@ class BootAPI:
                 # perms_ok is False
                 return
 
-            # FIMXE: conslidate into 1 server instance
+            # FIXME: conslidate into 1 server instance
 
             self.selinux_enabled = utils.is_selinux_enabled()
             self.dist = utils.check_dist()
@@ -124,11 +127,20 @@ class BootAPI:
 
             BootAPI.__has_loaded   = True
 
+            # load the modules first, or nothing else works...
             module_loader.load_modules()
 
-            self._config         = config.Config(self)
+            self._config = config.Config(self)
             self.deserialize()
 
+            # import signatures
+            if not utils.load_signatures(self.settings().signature_path):
+                return
+            else:
+                self.log("%d breeds and %d OS versions read from the signature file" % ( \
+                         len(utils.get_valid_breeds()), \
+                         len(utils.get_valid_os_versions())))
+
             self.authn = self.get_module_from_file(
                 "authentication",
                 "module",
@@ -139,7 +151,7 @@ class BootAPI:
                 "module",
                 "authz_allowall"
             )
-        
+
             # FIXME: pass more loggers around, and also see that those
             # using things via tasks construct their own kickgen/yumgen/
             # pxegen versus reusing this one, which has the wrong logger
@@ -181,11 +193,9 @@ class BootAPI:
         API instance, regardless of the serializer type.
         """
         if not os.path.exists("/var/lib/cobbler/.mtime"):
-            old = os.umask(0x777)
-            fd = open("/var/lib/cobbler/.mtime","w")
-            fd.write("0")
-            fd.close()
-            os.umask(old)
+            fd = os.open("/var/lib/cobbler/.mtime", os.O_CREAT|os.O_RDWR, 0200)
+            os.write(fd, "0")
+            os.close(fd)
             return 0
         fd = open("/var/lib/cobbler/.mtime")
         data = fd.read().strip()
@@ -222,7 +232,7 @@ class BootAPI:
         fd = open("/etc/cobbler/version")
         ydata = fd.read()
         fd.close()
-        data = yaml.load(ydata)
+        data = yaml.safe_load(ydata)
         if not extended:
             # for backwards compatibility and use with koan's comparisons
             elems = data["version_tuple"] 
@@ -570,6 +580,35 @@ class BootAPI:
 
     # ==========================================================================
 
+    def get_signatures(self):
+        return utils.SIGNATURE_CACHE
+
+    def signature_update(self, logger):
+        try:
+            tmpfile = tempfile.NamedTemporaryFile()
+            response = urllib2.urlopen(self.settings().signature_url)
+            sigjson = response.read()
+            tmpfile.write(sigjson)
+            tmpfile.flush()
+
+            logger.debug("Successfully got file from %s" % self.settings().signature_url)
+            # test the import without caching it
+            if not utils.load_signatures(tmpfile.name,cache=False):
+                logger.error("Downloaded signatures failed test load (tempfile = %s)" % tmpfile.name)
+                return False
+
+            # rewrite the real signature file and import it for real
+            f = open(self.settings().signature_path,"w")
+            f.write(sigjson)
+            f.close()
+
+            return utils.load_signatures(self.settings().signature_path)
+        except:
+            utils.log_exc(logger)
+            return False
+
+    # ==========================================================================
+
     def dump_vars(self, obj, format=False):
         return obj.dump_vars(format)
 
@@ -646,6 +685,33 @@ class BootAPI:
 
     # ==========================================================================
 
+    def generate_gpxe(self,profile,system):
+        self.log("generate_gpxe")
+        if system:
+            return self.pxegen.generate_gpxe("system",system)
+        else:
+            return self.pxegen.generate_gpxe("profile",profile)
+
+    # ==========================================================================
+
+    def generate_bootcfg(self,profile,system):
+        self.log("generate_bootcfg")
+        if system:
+            return self.pxegen.generate_bootcfg("system",system)
+        else:
+            return self.pxegen.generate_bootcfg("profile",profile)
+
+    # ==========================================================================
+
+    def generate_script(self,profile,system,name):
+        self.log("generate_script")
+        if system:
+            return self.pxegen.generate_script("system",system,name)
+        else:
+            return self.pxegen.generate_script("profile",profile,name)
+
+    # ==========================================================================
+
     def check(self, logger=None):
         """
         See if all preqs for network booting are valid.  This returns
@@ -702,6 +768,15 @@ class BootAPI:
 
     # ==========================================================================
 
+    def sync_dhcp(self, verbose=False, logger=None):
+        """
+        Only build out the DHCP configuration
+        """
+        self.log("sync_dhcp")
+        sync = self.get_sync(verbose=verbose, logger=logger)
+        return sync.sync_dhcp()
+    # ==========================================================================
+
     def get_sync(self,verbose=False,logger=None):
         self.dhcp = self.get_module_from_file(
            "dhcp",
@@ -794,6 +869,12 @@ class BootAPI:
             if rsync_flags:
                 rsync_cmd = rsync_cmd + " " + rsync_flags
 
+            # if --available-as was specified, limit the files we 
+            # pull down via rsync to just those that are critical
+            # to detecting what the distro is
+            if network_root is not None:
+                rsync_cmd = rsync_cmd + " --include-from=/etc/cobbler/import_rsync_whitelist"
+
             # kick off the rsync now
             utils.run_this(rsync_cmd, (spacer, mirror_url, path), self.logger)
 
@@ -824,22 +905,24 @@ class BootAPI:
                     self.log("Network root given to --available-as is missing a colon, please see the manpage example.")
                     return False
 
-        importer_modules = self.get_modules_in_category("manage/import")
-        for importer_module in importer_modules:
-            manager = importer_module.get_import_manager(self._config,logger)
-            try:
-                (found,pkgdir) = manager.check_for_signature(path,breed)
-                if found: 
-                    self.log("running import manager: %s" % manager.what())
-                    return manager.run(pkgdir,mirror_name,path,network_root,kickstart_file,rsync_flags,arch,breed,os_version)
-            except:
-                self.log("an exception occured while running the import manager")
-                self.log("error was: %s" % sys.exc_info()[1])
-                continue
-        self.log("No import managers found a valid signature at the location specified")
-        # FIXME: since we failed, we should probably remove the 
-        # path tree we created above so we don't leave cruft around
-        return False
+        #importer_modules = self.get_modules_in_category("manage/import")
+        #for importer_module in importer_modules:
+        #    manager = importer_module.get_import_manager(self._config,logger)
+        #    try:
+        #        (found,pkgdir) = manager.check_for_signature(path,breed)
+        #        if found: 
+        #            self.log("running import manager: %s" % manager.what())
+        #            return manager.run(pkgdir,mirror_name,path,network_root,kickstart_file,rsync_flags,arch,breed,os_version)
+        #    except:
+        #        self.log("an exception occured while running the import manager")
+        #        self.log("error was: %s" % sys.exc_info()[1])
+        #        continue
+        #self.log("No import managers found a valid signature at the location specified")
+        ## FIXME: since we failed, we should probably remove the 
+        ## path tree we created above so we don't leave cruft around
+        #return False
+        import_module = self.get_module_by_name("manage_import_signatures").get_import_manager(self._config,logger)
+        return import_module.run(path,mirror_name,network_root,kickstart_file,arch,breed,os_version)
 
     # ==========================================================================
 
@@ -905,6 +988,13 @@ class BootAPI:
         """
         return module_loader.get_module_from_file(section,name,fallback)
 
+    def get_module_name_from_file(self,section,name,fallback=None):
+        """
+        Looks up a module the same as get_module_from_file but returns
+        the module name rather than the module itself
+        """
+        return module_loader.get_module_from_file(section,name,fallback,just_name=True)
+
     def get_modules_in_category(self,category):
         """
         Returns all modules in a given category, for instance "serializer", or "cli".
@@ -934,10 +1024,10 @@ class BootAPI:
 
     # ==========================================================================
 
-    def build_iso(self,iso=None,profiles=None,systems=None,buildisodir=None,distro=None,standalone=None,source=None, exclude_dns=None, logger=None):
+    def build_iso(self,iso=None,profiles=None,systems=None,buildisodir=None,distro=None,standalone=None,source=None, exclude_dns=None, mkisofs_opts=None, logger=None):
         builder = action_buildiso.BuildIso(self._config, logger=logger)
         return builder.run(
-           iso=iso, profiles=profiles, systems=systems, buildisodir=buildisodir, distro=distro, standalone=standalone, source=source, exclude_dns=exclude_dns
+           iso=iso, profiles=profiles, systems=systems, buildisodir=buildisodir, distro=distro, standalone=standalone, source=source, exclude_dns=exclude_dns, mkisofs_opts=mkisofs_opts
         )
 
     # ==========================================================================
@@ -1015,7 +1105,6 @@ class BootAPI:
         """
         return action_power.PowerTool(self._config, system, self, user, password, logger = logger).power("status")
 
-
     # ==========================================================================
 
     def clear_logs(self, system, logger=None):
