@@ -1,8 +1,8 @@
 """
 Command line interface for cobbler.
 
-Copyright 2006-2009, Red Hat, Inc
-Michael DeHaan <mdehaan@redhat.com>
+Copyright 2006-2009, Red Hat, Inc and Others
+Michael DeHaan <michael.dehaan AT gmail>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -38,8 +38,9 @@ import item_image
 import item_mgmtclass
 import item_package
 import item_file
+import settings
 
-OBJECT_ACTIONS   = {
+OBJECT_ACTIONS_MAP   = {
    "distro"    : "add copy edit find list remove rename report".split(" "),
    "profile"   : "add copy dumpvars edit find getks list remove rename report".split(" "),
    "system"    : "add copy dumpvars edit find getks list remove rename report poweron poweroff powerstatus reboot".split(" "),
@@ -48,23 +49,92 @@ OBJECT_ACTIONS   = {
    "mgmtclass" : "add copy edit find list remove rename report".split(" "),
    "package"   : "add copy edit find list remove rename report".split(" "),
    "file"      : "add copy edit find list remove rename report".split(" "),
+   "setting"   : "edit report".split(" "),
+   "signature" : "report update".split(" "),
 } 
-OBJECT_TYPES = OBJECT_ACTIONS.keys()
+OBJECT_TYPES = OBJECT_ACTIONS_MAP.keys()
+# would like to use from_iterable here, but have to support python 2.4
+OBJECT_ACTIONS = []
+for actions in OBJECT_ACTIONS_MAP.values():
+   OBJECT_ACTIONS += actions
 DIRECT_ACTIONS = "aclsetup buildiso import list replicate report reposync sync validateks version".split()
 
 ####################################################
 
 def report_items(remote, otype):
-   items = remote.get_items(otype)
-   for x in items:
-      report_item(remote,otype,item=x)
+   if otype == "setting":
+       items = remote.get_settings()
+       keys = items.keys()
+       keys.sort()
+       for key in keys:
+           item = {'name':key, 'value':items[key]}
+           report_item(remote,otype,item=item)
+   elif otype == "signature":
+       items = remote.get_signatures()
+       total_breeds = 0
+       total_sigs = 0
+       if items.has_key("breeds"):
+           print "Currently loaded signatures:"
+           bkeys = items["breeds"].keys()
+           bkeys.sort()
+           total_breeds = len(bkeys)
+           for breed in bkeys:
+               print "%s:" % breed
+               oskeys = items["breeds"][breed].keys()
+               oskeys.sort()
+               if len(oskeys) > 0:
+                   total_sigs += len(oskeys)
+                   for osversion in oskeys:
+                       print "\t%s" % osversion
+               else:
+                   print "\t(none)"
+           print "\n%d breeds with %d total signatures loaded" % (total_breeds,total_sigs)
+       else:
+           print "No breeds found in the signature, a signature update is recommended"
+           sys.exit(1)
+   else:
+       items = remote.get_items(otype)
+       for x in items:
+           report_item(remote,otype,item=x)
 
 def report_item(remote,otype,item=None,name=None):
    if item is None:
-      item = remote.get_item(otype, name)
-      if item == "~":
-          print "No %s found: %s" % (otype, name)
-          sys.exit(1)
+      if otype == "setting":
+          cur_settings = remote.get_settings()
+          try:
+              item = {'name':name, 'value':cur_settings[name]}
+          except:
+              print "Setting not found: %s" % name
+              sys.exit(1)
+      elif otype == "signature":
+          items = remote.get_signatures()
+          total_sigs = 0
+          if items.has_key("breeds"):
+              print "Currently loaded signatures:"
+              if items["breeds"].has_key(name):
+                  print "%s:" % name
+                  oskeys = items["breeds"][name].keys()
+                  oskeys.sort()
+                  if len(oskeys) > 0:
+                      total_sigs += len(oskeys)
+                      for osversion in oskeys:
+                          print "\t%s" % osversion
+                  else:
+                      print "\t(none)"
+                  print "\nBreed '%s' has %d total signatures" % (name,total_sigs)
+              else:
+                  print "No breed named '%s' found" % name
+                  sys.exit(1)
+          else:
+              print "No breeds found in the signature, a signature update is recommended"
+              sys.exit(1)
+          return
+      else:
+          item = remote.get_item(otype, name)
+          if item == "~":
+              print "No %s found: %s" % (otype, name)
+              sys.exit(1)
+
    if otype == "distro":
       data = utils.printable_from_fields(item, item_distro.FIELDS)
    elif otype == "profile":
@@ -81,6 +151,8 @@ def report_item(remote,otype,item=None,name=None):
       data = utils.printable_from_fields(item,item_package.FIELDS)
    elif otype == "file":
       data = utils.printable_from_fields(item,item_file.FIELDS)
+   elif otype == "setting":
+      data = "%-40s: %s" % (item['name'],item['value'])
    print data
 
 def list_items(remote,otype):
@@ -142,7 +214,7 @@ class BootCLI:
         """
         if object_type is None or len(args) < 3:
             return None
-        if args[2] in OBJECT_ACTIONS[object_type]:
+        if args[2] in OBJECT_ACTIONS_MAP[object_type]:
             return args[2]
         return None
 
@@ -179,7 +251,8 @@ class BootCLI:
         try:
             s.ping()
         except:
-            print >> sys.stderr, "httpd does not appear to be running and proxying cobbler"
+            print >> sys.stderr, "httpd does not appear to be running and proxying cobbler, or SELinux is in the way. Original traceback:"
+            traceback.print_exc()
             sys.exit(411)
 
         if not os.path.exists("/var/lib/cobbler/web.ss"):
@@ -258,6 +331,8 @@ class BootCLI:
             return item_package.FIELDS
         elif object_type == "file":
             return item_file.FIELDS
+        elif object_type == "setting":
+            return settings.FIELDS
 
     def object_command(self, object_type, object_action):
         """
@@ -266,41 +341,16 @@ class BootCLI:
         task_id = -1 # if assigned, we must tail the logfile
         
         fields = self.get_fields(object_type)
-        if object_action in [ "add", "edit", "copy", "rename", "find" ]:
+        if object_action in [ "add", "edit", "copy", "rename", "find", "remove" ]:
             utils.add_options_from_fields(object_type, self.parser, fields, object_action)
         elif object_action in [ "list" ]:
             pass
-        else:
+        elif object_action != "update":
             self.parser.add_option("--name", dest="name", help="name of object")
         (options, args) = self.parser.parse_args()
 
-        if object_action in [ "add", "edit", "copy", "rename", "remove", "reboot" ]:
-            if opt(options, "name") == "":
-                print "--name is required"
-                sys.exit(1)
-            self.remote.xapi_object_edit(object_type, options.name, object_action, utils.strip_none(vars(options), omit_none=True), self.token)
-        elif object_action == "getks":
-            if object_type == "profile":
-                data = self.remote.generate_kickstart(options.name,"")
-            elif object_type == "system":
-                data = self.remote.generate_kickstart("",options.name)
-            print data
-        elif object_action == "dumpvars":
-            if object_type == "profile":
-                data = self.remote.get_blended_data(options.name,"")
-            elif object_type == "system":
-                data = self.remote.get_blended_data("",options.name)
-            # FIXME: pretty-printing and sorting here
-            keys = data.keys()
-            keys.sort()
-            for x in keys:
-               print "%s : %s" % (x, data[x])
-        elif object_action in [ "poweron", "poweroff", "powerstatus", "reboot" ]:
-            power={}
-            power["power"] = object_action.replace("power","")
-            power["systems"] = [options.name]
-            task_id = self.remote.background_power_system(power, self.token)
-        elif object_action == "report":
+        # the first three don't require a name
+        if object_action == "report":
             if options.name is not None:
                 report_item(self.remote,object_type,None,options.name)
             else:
@@ -311,6 +361,56 @@ class BootCLI:
             items = self.remote.find_items(object_type, utils.strip_none(vars(options), omit_none=True), "name", False)
             for item in items:
                 print item
+        elif object_action in OBJECT_ACTIONS:
+            if opt(options, "name") == "" and object_action != "update":
+                print "--name is required"
+                sys.exit(1)
+            if object_action in [ "add", "edit", "copy", "rename", "remove" ]:
+                try:
+                    if object_type == "setting":        
+                        settings = self.remote.get_settings()
+                        if options.value == None:
+                            raise RuntimeError("You must specify a --value when editing a setting")
+                        elif not settings.get('allow_dynamic_settings',False):
+                            raise RuntimeError("Dynamic settings changes are not enabled. Change the allow_dynamic_settings to 1 and restart cobblerd to enable dynamic settings changes")
+                        elif options.name == 'allow_dynamic_settings':
+                            raise RuntimeError("Cannot modify that setting live")
+                        elif self.remote.modify_setting(options.name,options.value,self.token):
+                            raise RuntimeError("Changing the setting failed")
+                    else:
+                        self.remote.xapi_object_edit(object_type, options.name, object_action, utils.strip_none(vars(options), omit_none=True), self.token)
+                except xmlrpclib.Fault, (err):
+                    (etype, emsg) = err.faultString.split(":",1)
+                    print emsg[1:-1] # don't print the wrapping quotes
+                    sys.exit(1)
+                except RuntimeError, (err):
+                    print err.args[0]
+                    sys.exit(1)
+            elif object_action == "getks":
+                if object_type == "profile":
+                    data = self.remote.generate_kickstart(options.name,"")
+                elif object_type == "system":
+                    data = self.remote.generate_kickstart("",options.name)
+                print data
+            elif object_action == "dumpvars":
+                if object_type == "profile":
+                    data = self.remote.get_blended_data(options.name,"")
+                elif object_type == "system":
+                    data = self.remote.get_blended_data("",options.name)
+                # FIXME: pretty-printing and sorting here
+                keys = data.keys()
+                keys.sort()
+                for x in keys:
+                   print "%s : %s" % (x, data[x])
+            elif object_action in [ "poweron", "poweroff", "powerstatus", "reboot" ]:
+                power={}
+                power["power"] = object_action.replace("power","")
+                power["systems"] = [options.name]
+                task_id = self.remote.background_power_system(power, self.token)
+            elif object_action == "update":
+                task_id = self.remote.background_signature_update(utils.strip_none(vars(options),omit_none=True), self.token)
+            else:
+                raise exceptions.NotImplementedError()
         else:
             raise exceptions.NotImplementedError() 
             
@@ -339,6 +439,7 @@ class BootCLI:
             self.parser.add_option("--standalone", dest="standalone", action="store_true", help="(OPTIONAL) creates a standalone ISO with all required distro files on it")
             self.parser.add_option("--source",   dest="source",   help="(OPTIONAL) used with --standalone to specify a source for the distribution files")
             self.parser.add_option("--exclude-dns", dest="exclude_dns", action="store_true", help="(OPTIONAL) prevents addition of name server addresses to the kernel boot options")
+            self.parser.add_option("--mkisofs-opts", dest="mkisofs_opts", help="(OPTIONAL) extra options for mkisofs")
 
             (options, args) = self.parser.parse_args()
             task_id = self.start_task("buildiso",options)
@@ -517,7 +618,7 @@ class BootCLI:
         """
         Prints the subcommands for a given object, e.g. "cobbler distro --help"
         """
-        commands = OBJECT_ACTIONS[object_type]
+        commands = OBJECT_ACTIONS_MAP[object_type]
         commands.sort()
         print "usage\n====="
         for c in commands:
