@@ -1,8 +1,8 @@
 """
 cobbler daemon for logging remote syslog traffic during kickstart
  
-Copyright 2007-2009, Red Hat, Inc
-Michael DeHaan <mdehaan@redhat.com>
+Copyright 2007-2009, Red Hat, Inc and Others
+Michael DeHaan <michael.dehaan AT gmail>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -58,20 +58,17 @@ def regen_ss_file():
     data = fd.read(512)
     fd.close()
     if not os.path.isfile(ssfile):
-        um = os.umask(int('0027',16))
-        fd = open(ssfile,"w+")
-        fd.write(binascii.hexlify(data))
-        fd.close()
-        os.umask(um)
-        utils.os_system("chmod 700 /var/lib/cobbler/web.ss")
+        fd = os.open(ssfile,os.O_CREAT|os.O_RDWR,0600)
+        os.write(fd,binascii.hexlify(data))
+        os.close(fd)
         http_user = "apache"
         if utils.check_dist() in [ "debian", "ubuntu" ]:
             http_user = "www-data"
         utils.os_system("chown %s /var/lib/cobbler/web.ss"%http_user )
     else:
-        fd = open(ssfile,"w+")
-        fd.write(binascii.hexlify(data))
-        fd.close()
+        fd = os.open(ssfile,os.O_CREAT|os.O_RDWR,0600)
+        os.write(fd,binascii.hexlify(data))
+        os.close(fd)
 
     return 1
 
@@ -130,16 +127,10 @@ def do_xmlrpc_rw(bootapi,settings,port):
 
 if __name__ == "__main__":
 
-    #main()
+    bootapi  = cobbler_api.BootAPI()
+    settings = bootapi.settings()
 
-    #bootapi      = cobbler_api.BootAPI()
-    #settings     = bootapi.settings()
-    #syslog_port  = settings.syslog_port
-    #xmlrpc_port  = settings.xmlrpc_port
-    #xmlrpc_port2 = settings.xmlrpc_rw_port
-    #logger       = bootapi.logger_remote
-    #do_xmlrpc_unix(bootapi, settings, logger)
-   
     regen_ss_file()
 
+    do_xmlrpc_rw(bootapi, settings, 25151)
 
