@@ -1,8 +1,8 @@
 """
 Base class for any serializable list of things...
 
-Copyright 2006-2009, Red Hat, Inc
-Michael DeHaan <mdehaan@redhat.com>
+Copyright 2006-2009, Red Hat, Inc and Others
+Michael DeHaan <michael.dehaan AT gmail>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -160,9 +160,9 @@ class Collection:
 
     def copy(self,ref,newname,logger=None): 
         ref       = ref.make_clone()
-        ref.name  = newname
         ref.uid   = self.config.generate_uid()
         ref.ctime = 0
+        ref.set_name(newname)
         if ref.COLLECTION_TYPE == "system":
             # this should only happen for systems
             for iname in ref.interfaces.keys():
@@ -189,6 +189,16 @@ class Collection:
 
         self.add(newref, with_triggers=with_triggers,save=True)
 
+        # for mgmt classes, update all objects that use it
+        if ref.COLLECTION_TYPE == "mgmtclass":
+            for what in ["distro","profile","system"]:
+                items = self.api.find_items(what,{"mgmt_classes":oldname})
+                for item in items:
+                    for i in range(0,len(item.mgmt_classes)):
+                        if item.mgmt_classes[i] == oldname:
+                            item.mgmt_classes[i] = newname
+                    self.api.add_item(what,item,save=True)
+
         # for a repo, rename the mirror directory
         if ref.COLLECTION_TYPE == "repo":
             path = "/var/www/cobbler/repo_mirror/%s" % ref.name
@@ -198,8 +208,15 @@ class Collection:
 
         # for a distro, rename the mirror and references to it
         if ref.COLLECTION_TYPE == 'distro':
-            path = "/var/www/cobbler/ks_mirror/%s" % ref.name
-            if os.path.exists(path):
+            path = utils.find_distro_path(self.api.settings(), ref)
+
+            # create a symlink for the new distro name
+            utils.link_distro(self.api.settings(), newref)
+
+            # test to see if the distro path is based directly
+            # on the name of the distro. If it is, things need
+            # to updated accordingly
+            if os.path.exists(path) and path == "/var/www/cobbler/ks_mirror/%s" % ref.name:
                 newpath = "/var/www/cobbler/ks_mirror/%s" % newref.name
                 os.renames(path, newpath)
 
@@ -310,8 +327,14 @@ class Collection:
 
             if with_sync:
                 if isinstance(ref, item_system.System):
+                    # we don't need openvz containers to be network bootable
+                    if ref.virt_type == "openvz":
+                        ref.netboot_enabled = False
                     self.lite_sync.add_single_system(ref.name)
                 elif isinstance(ref, item_profile.Profile):
+                    # we don't need openvz containers to be network bootable
+                    if ref.virt_type == "openvz":
+                        ref.enable_menu = 0
                     self.lite_sync.add_single_profile(ref.name) 
                 elif isinstance(ref, item_distro.Distro):
                     self.lite_sync.add_single_distro(ref.name)
