@@ -2,8 +2,8 @@
 A distro represents a network bootable matched set of kernels
 and initrd files
 
-Copyright 2006-2009, Red Hat, Inc
-Michael DeHaan <mdehaan@redhat.com>
+Copyright 2006-2009, Red Hat, Inc and Others
+Michael DeHaan <michael.dehaan AT gmail>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -50,7 +50,7 @@ class Distros(collection.Collection):
         # first see if any Groups use this distro
         if not recursive:
             for v in self.config.profiles():
-                if v.distro.lower() == name:
+                if v.distro and v.distro.lower() == name:
                     raise CX(_("removal would orphan profile: %s") % v.name)
 
         obj = self.find(name=name)
@@ -80,7 +80,8 @@ class Distros(collection.Collection):
 
             # look through all mirrored directories and find if any directory is holding
             # this particular distribution's kernel and initrd
-            possible_storage = glob.glob("/var/www/cobbler/ks_mirror/*")
+            settings = self.config.settings()
+            possible_storage = glob.glob(settings.webdir+"/ks_mirror/*")
             path = None
             for storage in possible_storage:
                 if os.path.dirname(obj.kernel).find(storage) != -1:
@@ -89,7 +90,7 @@ class Distros(collection.Collection):
 
             # if we found a mirrored path above, we can delete the mirrored storage /if/
             # no other object is using the same mirrored storage.
-            if with_delete and path is not None and os.path.exists(path) and kernel.find("/var/www/cobbler") != -1:
+            if with_delete and path is not None and os.path.exists(path) and kernel.find(settings.webdir) != -1:
                # this distro was originally imported so we know we can clean up the associated
                # storage as long as nothing else is also using this storage.
                found = False
