@@ -3,8 +3,8 @@ Repositories in cobbler are way to create a local mirror of a yum repository.
 When used in conjunction with a mirrored kickstart tree (see "cobbler import")
 outside bandwidth needs can be reduced and/or eliminated.
 
-Copyright 2006-2009, Red Hat, Inc
-Michael DeHaan <mdehaan@redhat.com>
+Copyright 2006-2009, Red Hat, Inc and Others
+Michael DeHaan <michael.dehaan AT gmail>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
