@@ -2,8 +2,8 @@
 Systems are hostnames/MACs/IP names and the associated profile
 they belong to.
 
-Copyright 2008-2009, Red Hat, Inc
-Michael DeHaan <mdehaan@redhat.com>
+Copyright 2008-2009, Red Hat, Inc and Others
+Michael DeHaan <michael.dehaan AT gmail>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
