@@ -1,8 +1,8 @@
 """
 An Item is a serializable thing that can appear in a Collection
 
-Copyright 2006-2009, Red Hat, Inc
-Michael DeHaan <mdehaan@redhat.com>
+Copyright 2006-2009, Red Hat, Inc and Others
+Michael DeHaan <michael.dehaan AT gmail>
 
 This software may be freely redistributed under the terms of the GNU
 general public license.
@@ -174,7 +174,7 @@ class Item:
         Kernel options are a space delimited list,
         like 'a=b c=d e=f g h i=j' or a hash.
         """
-        (success, value) = utils.input_string_or_hash(options)
+        (success, value) = utils.input_string_or_hash(options,allow_multiples=True)
         if not success:
             raise CX(_("invalid kernel options"))
         else:
@@ -193,7 +193,7 @@ class Item:
         Post kernel options are a space delimited list,
         like 'a=b c=d e=f g h i=j' or a hash.
         """
-        (success, value) = utils.input_string_or_hash(options)
+        (success, value) = utils.input_string_or_hash(options,allow_multiples=True)
         if not success:
             raise CX(_("invalid post kernel options"))
         else:
@@ -213,7 +213,7 @@ class Item:
         The meta tags are used as input to the templating system
         to preprocess kickstart files
         """
-        (success, value) = utils.input_string_or_hash(options,allow_multiples=False)
+        (success, value) = utils.input_string_or_hash(options,allow_multiples=True)
         if not success:
             return False
         else:
@@ -245,7 +245,7 @@ class Item:
             self.mgmt_parameters = mgmt_parameters
         else:
             import yaml
-            data = yaml.load(mgmt_parameters)
+            data = yaml.safe_load(mgmt_parameters)
             if type(data) is not dict:
                 raise CX(_("Input YAML in Puppet Parameter field must evaluate to a dictionary."))
             self.mgmt_parameters = data
@@ -271,6 +271,26 @@ class Item:
                 self.template_files = value
             return True
 
+    def set_boot_files(self,boot_files,inplace=False):
+        """
+        A comma seperated list of req_name=source_file_path
+        that should be fetchable via tftp 
+        """
+        (success, value) = utils.input_string_or_hash(boot_files,allow_multiples=False)
+        if not success:
+            return False
+        else:
+            if inplace:
+                for key in value.keys():
+                    if key.startswith("~"):
+                        del self.boot_files[key[1:]]
+                    else:
+                        self.boot_files[key] = value[key]
+            else:
+                self.boot_files= value
+            return True
+
+
     def set_fetchable_files(self,fetchable_files,inplace=False):
         """
         A comma seperated list of virt_name=path_to_template
@@ -314,10 +334,16 @@ class Item:
         # special case for systems
         key_found_already = False
         if data.has_key("interfaces"):
-            if key in [ "mac_address", "ip_address", "subnet", "virt_bridge", "dhcp_tag", "dns_name", "static_routes", "bonding", "bonding_opts", "bonding_master" ]:
+            if key in [ "mac_address", "ip_address", "subnet", "netmask", "virt_bridge", \
+                        "dhcp_tag", "dns_name", "static_routes", "interface_type", \
+                        "interface_master", "bonding_opts", "bridge_opts", "bonding", "bonding_master" ]:
+                if key == "bonding":
+                    key = "interface_type" # bonding is deprecated
+                elif key == "bonding_master":
+                    key = "interface_master" # bonding_master is deprecated
                 key_found_already = True
                 for (name, interface) in data["interfaces"].iteritems(): 
-                    if value is not None:
+                    if value is not None and interface.has_key(key):
                         if self.__find_compare(interface[key], value):
                             return True
 
