@@ -2,8 +2,8 @@
 A cobbler distribution.  A distribution is a kernel, and initrd, and potentially
 some kernel options.
 
-Copyright 2006-2009, Red Hat, Inc
-Michael DeHaan <mdehaan@redhat.com>
+Copyright 2006-2009, Red Hat, Inc and Others
+Michael DeHaan <michael.dehaan AT gmail>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -105,16 +105,15 @@ FIELDS = [
    [ "kernel_options",{},0,"Kernel Options",True,"Ex: selinux=permissive",0,"dict"],
    [ "kernel_options_post",{},0,"Kernel Options (Post Install)",True,"Ex: clocksource=pit noapic",0,"dict"],
    [ "ks_meta",{},0,"Kickstart Metadata",True,"Ex: dog=fang agent=86", 0,"dict"],
-   [ "arch",'i386',0,"Architecture",True,"", ['i386','x86_64','ia64','ppc','s390'],"str"],
-   [ "breed",'redhat',0,"Breed",True,"What is the type of distribution?",codes.VALID_OS_BREEDS,"str"],
-   [ "os_version","generic26",0,"OS Version",True,"Needed for some virtualization optimizations",codes.get_all_os_versions(),"str"],
+   [ "arch",'i386',0,"Architecture",True,"", ['i386','x86_64','ia64','ppc','ppc64','s390', 'arm'],"str"],
+   [ "breed",'redhat',0,"Breed",True,"What is the type of distribution?",utils.get_valid_breeds(),"str"],
+   [ "os_version","generic26",0,"OS Version",True,"Needed for some virtualization optimizations",utils.get_valid_os_versions(),"str"],
    [ "source_repos",[],0,"Source Repos", False,"",0,"list"],
    [ "depth",0,0,"Depth",False,"",0,"int"],
    [ "comment","",0,"Comment",True,"Free form text description",0,"str"],
    [ "tree_build_time",0,0,"Tree Build Time",False,"",0,"str"],
-   [ "network_widget_c","",0,"Add Management Class",True,"",0,"str"], # not a real field, a marker for the web app
    [ "mgmt_classes",[],0,"Management Classes",True,"Management classes for external config management",0,"list"],
-   [ "network_widget_d","",0,"Remove Selected Classes",True,"",0,"str"], # not a real field, a marker for the web app
+   [ "boot_files",{},0,"TFTP Boot Files",True,"Files copied into tftpboot beyond the kernel/initrd",0,"list"],
    [ "fetchable_files",{},0,"Fetchable Files",True,"Templates for tftp or wget",0,"list"],
    [ "template_files",{},0,"Template Files",True,"File mappings for built-in config management",0,"list"],
    [ "redhat_management_key","<<inherit>>",0,"Red Hat Management Key",True,"Registration key for RHN, Spacewalk, or Satellite",0,"str"],
@@ -164,7 +163,7 @@ class Distro(item.Item):
 
     def set_tree_build_time(self, datestamp):
         """
-        Sets the import time of the distro, for use by action_import.py.
+        Sets the import time of the distro.
         If not imported, this field is not meaningful.
         """
         self.tree_build_time = float(datestamp)
@@ -233,7 +232,7 @@ class Distro(item.Item):
 
         if utils.file_is_remote(self.kernel):
             if not utils.remote_file_exists(self.kernel):
-                raise CX("Error with distro %s - kernel not found" % (self.name))
+                raise CX("Error with distro %s - kernel '%s' not found" % (self.name,self.kernel))
         elif not os.path.exists(self.kernel):
             raise CX("Error with distro %s - kernel not found" % (self.name))
 
