@@ -2,8 +2,8 @@
 A Cobbler Image.  Tracks a virtual or physical image, as opposed to a answer
 file (kickstart) led installation.
 
-Copyright 2006-2009, Red Hat, Inc
-Michael DeHaan <mdehaan@redhat.com>
+Copyright 2006-2009, Red Hat, Inc and Others
+Michael DeHaan <michael.dehaan AT gmail>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 02110-1301  USA
 """
 
+import string
 import utils
 import item
 import time
@@ -32,16 +33,16 @@ from utils import _
 
 FIELDS = [
   ['name','',0,"Name",True,"",0,"str"],
-  ['arch','i386',0,"Architecture",True,"",["i386","x86_64","ia64","s390","ppc"],"str"],
-  ['breed','redhat',0,"Breed",True,"",codes.VALID_OS_BREEDS,"str"],
+  ['arch','i386',0,"Architecture",True,"",["i386","x86_64","ia64","s390","ppc","ppc64", "arm"],"str"],
+  ['breed','redhat',0,"Breed",True,"",utils.get_valid_breeds(),"str"],
   ['comment','',0,"Comment",True,"Free form text description",0,"str"],
   ['ctime',0,0,"",False,"",0,"float"],
   ['mtime',0,0,"",False,"",0,"float"],
   ['file','',0,"File",True,"Path to local file or nfs://user@host:path",0,"str"],
   ['depth',0,0,"",False,"",0,"int"],
-  ['image_type',"iso",0,"Image Type",True,"", ["iso","direct","virt-image"],"str"], #FIXME:complete?
+  ['image_type',"iso",0,"Image Type",True,"", ["iso","direct","memdisk","virt-image"],"str"], #FIXME:complete?
   ['network_count',1,0,"Virt NICs",True,"",0,"int"],
-  ['os_version','',0,"OS Version",True,"ex: rhel4",codes.get_all_os_versions(),"str"],
+  ['os_version','',0,"OS Version",True,"ex: rhel4",utils.get_valid_os_versions(),"str"],
   ['owners',"SETTINGS:default_ownership",0,"Owners",True,"Owners list for authz_ownership (space delimited)",[],"list"],
   ['parent','',0,"",False,"",0,"str"],
   ['kickstart','',0,"Kickstart",True,"Path to kickstart/answer file template",0,"str"],
@@ -52,7 +53,7 @@ FIELDS = [
   ["virt_disk_driver","SETTINGS:default_virt_disk_driver",0,"Virt Disk Driver Type",True,"The on-disk format for the virtualization disk","raw","str"],
   ['virt_path','',0,"Virt Path",True,"Ex: /directory or VolGroup00",0,"str"],
   ['virt_ram',"SETTINGS:default_virt_ram",0,"Virt RAM (MB)",True,"",0,"int"],
-  ['virt_type',"SETTINGS:default_virt_type",0,"Virt Type",True,"",["xenpv","xenfv","qemu","vmware"],"str"],
+  ['virt_type',"SETTINGS:default_virt_type",0,"Virt Type",True,"",["xenpv","xenfv","qemu","kvm", "vmware"],"str"],
   ['uid',"",0,"",False,"",0,"str"]
 ]
 
@@ -87,7 +88,8 @@ class Image(item.Item):
         if kickstart is None or kickstart == "" or kickstart == "delete":
             self.kickstart = ""
             return True
-        if utils.find_kickstart(kickstart):
+        kickstart = utils.find_kickstart(kickstart)
+        if kickstart:
             self.kickstart = kickstart
             return True
         raise CX(_("kickstart not found for image"))
@@ -155,8 +157,8 @@ class Image(item.Item):
         virt-clone = a cloned virtual disk (FIXME: not yet supported), virtual only
         memdisk    = hdd image (physical only)
         """
-        if not image_type in [ "direct", "iso", "memdisk", "virt-clone" ]:
-           raise CX(_("image type must be 'direct', 'iso', or 'virt-clone'"))
+        if not image_type in self.get_valid_image_types():
+           raise CX(_("image type must be on of the following: %s") % string.join(self.get_valid_image_types(),", "))
         self.image_type = image_type
         return True
 
@@ -193,6 +195,9 @@ class Image(item.Item):
     def set_virt_path(self,path):
         return utils.set_virt_path(self,path)
 
+    def get_valid_image_types(self):
+        return ["direct","iso","memdisk","virt-clone"]
+
     def get_parent(self):
         """
         Return object next highest up the tree.
