@@ -1,8 +1,8 @@
 """
 A Cobbler System.
 
-Copyright 2006-2009, Red Hat, Inc
-Michael DeHaan <mdehaan@redhat.com>
+Copyright 2006-2009, Red Hat, Inc and Others
+Michael DeHaan <michael.dehaan AT gmail>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@ import item
 import time
 from cexceptions import *
 from utils import _
+import sys
 
 # this datastructure is described in great detail in item_distro.py -- read the comments there.
 
@@ -34,27 +35,32 @@ FIELDS = [
   ["owners","SETTINGS:default_ownership",0,"Owners",True,"Owners list for authz_ownership (space delimited)",0,"list"],
   ["profile",None,0,"Profile",True,"Parent profile",[],"str"],
   ["image",None,0,"Image",True,"Parent image (if not a profile)",0,"str"],
+  ["status","production",0,"Status",True,"System status",["development","testing","acceptance","production"],"str"],
   ["kernel_options",{},0,"Kernel Options",True,"Ex: selinux=permissive",0,"dict"],
   ["kernel_options_post",{},0,"Kernel Options (Post Install)",True,"Ex: clocksource=pit noapic",0,"dict"],
   ["ks_meta",{},0,"Kickstart Metadata",True,"Ex: dog=fang agent=86",0,"dict"],
+  ["enable_gpxe","SETTINGS:enable_gpxe",0,"Enable gPXE?",True,"Use gPXE instead of PXELINUX for advanced booting options",0,"bool"],
+  ["proxy","<<inherit>>",0,"Proxy",True,"Proxy URL",0,"str"],
   ["netboot_enabled",True,0,"Netboot Enabled",True,"PXE (re)install this machine at next boot?",0,"bool"],
   ["kickstart","<<inherit>>",0,"Kickstart",True,"Path to kickstart template",0,"str"],
   ["comment","",0,"Comment",True,"Free form text description",0,"str"],
   ["depth",2,0,"",False,"",0,"int"],
   ["server","<<inherit>>",0,"Server Override",True,"See manpage or leave blank",0,"str"],
   ["virt_path","<<inherit>>",0,"Virt Path",True,"Ex: /directory or VolGroup00",0,"str"],
-  ["virt_type","<<inherit>>",0,"Virt Type",True,"Virtualization technology to use",["xenpv","xenfv","qemu","vmware"],"str"],
+  ["virt_type","<<inherit>>",0,"Virt Type",True,"Virtualization technology to use",["xenpv","xenfv","qemu","kvm","vmware","openvz"],"str"],
   ["virt_cpus","<<inherit>>",0,"Virt CPUs",True,"",0,"int"],
   ["virt_file_size","<<inherit>>",0,"Virt File Size(GB)",True,"",0,"float"],
+  ["virt_disk_driver","<<inherit>>",0,"Virt Disk Driver Type",True,"The on-disk format for the virtualization disk","raw","str"],
   ["virt_ram","<<inherit>>",0,"Virt RAM (MB)",True,"",0,"int"],
   ["virt_auto_boot","<<inherit>>",0,"Virt Auto Boot",True,"Auto boot this VM?",0,"bool"],
+  ["virt_pxe_boot",0,0,"Virt PXE Boot",True,"Use PXE to build this VM?",0,"bool"],
   ["ctime",0,0,"",False,"",0,"float"],
   ["mtime",0,0,"",False,"",0,"float"],
-  ["power_type","SETTINGS:power_management_default_type",0,"Power Management Type",True,"",utils.get_power_types(),"str"],
+  ["power_type","SETTINGS:power_management_default_type",0,"Power Management Type",True,"Power management script to use",utils.get_power_types(),"str"],
   ["power_address","",0,"Power Management Address",True,"Ex: power-device.example.org",0,"str"],
-  ["power_user","",0,"Power Username ",True,"",0,"str"],
-  ["power_pass","",0,"Power Password",True,"",0,"str"],
-  ["power_id","",0,"Power ID",True,"Usually a plug number or blade name, if power type requires it",0,"str"],
+  ["power_user","",0,"Power Management Username ",True,"",0,"str"],
+  ["power_pass","",0,"Power Management Password",True,"",0,"str"],
+  ["power_id","",0,"Power Management ID",True,"Usually a plug number or blade name, if power type requires it",0,"str"],
   ["hostname","",0,"Hostname",True,"",0,"str"],
   ["gateway","",0,"Gateway",True,"",0,"str"],
   ["name_servers",[],0,"Name Servers",True,"space delimited",0,"list"],
@@ -64,23 +70,26 @@ FIELDS = [
   ["network_widget_a","",0,"Add Interface",True,"",0,"str"], # not a real field, a marker for the web app
   ["network_widget_b","",0,"Edit Interface",True,"",0,"str"], # not a real field, a marker for the web app
   ["*mac_address","",0,"MAC Address",True,"(Place \"random\" in this field for a random MAC Address.)",0,"str"],
+  ["network_widget_c","",0,"",True,"",0,"str"], # not a real field, a marker for the web app
   ["*mtu","",0,"MTU",True,"",0,"str"],
-  ["*ip_address","",0,"IP Address",True,"",0,"str"],
-  ["*bonding","na",0,"Bonding Mode",True,"",["na","master","slave"],"str"],
-  ["*bonding_master","",0,"Bonding Master",True,"",0,"str"],
-  ["*bonding_opts","",0,"Bonding Opts",True,"",0,"str"],
-  ["*management",False,0,"Management Interface",True,"Is this the management interface?",0,"bool"],
-  ["*static",False,0,"Static",True,"Is this interface static?",0,"bool"],
-  ["*subnet","",0,"Subnet",True,"",0,"str"],
-  ["*dhcp_tag","",0,"DHCP Tag",True,"",0,"str"],
-  ["*dns_name","",0,"DNS Name",True,"",0,"str"],
-  ["*static_routes",[],0,"Static Routes",True,"",0,"list"],
-  ["*virt_bridge","",0,"Virt Bridge",True,"",0,"str"],
-  ["*ipv6_address","",0,"IPv6 Address",True,"",0,"str"],
-  ["*ipv6_secondaries",[],0,"IPv6 Secondaries",True,"space delimited",0,"list"],
-  ["*ipv6_mtu","",0,"IPv6 MTU",True,"",0,"str"],
-  ["*ipv6_static_routes",[],0,"IPv6 Static Routes",True,"",0,"list"],
-  ["*ipv6_default_gateway","",0,"IPv6 Default Gateway",True,"",0,"str"],
+  ["*ip_address","",0,"IP Address",True,"Should be used with --interface",0,"str"],
+  ["*interface_type","na",0,"Interface Type",True,"Should be used with --interface",["na","master","slave","bond","bond_slave","bridge","bridge_slave","bonded_bridge_slave"],"str"],
+  ["*interface_master","",0,"Master Interface",True,"Should be used with --interface",0,"str"],
+  ["*bonding_opts","",0,"Bonding Opts",True,"Should be used with --interface",0,"str"],
+  ["*bridge_opts","",0,"Bridge Opts",True,"Should be used with --interface",0,"str"],
+  ["*management",False,0,"Management Interface",True,"Is this the management interface? Should be used with --interface",0,"bool"],
+  ["*static",False,0,"Static",True,"Is this interface static? Should be used with --interface",0,"bool"],
+  ["*netmask","",0,"Subnet Mask",True,"Should be used with --interface",0,"str"],
+  ["*if_gateway","",0,"Per-Interface Gateway",True,"Should be used with --interface",0,"str"],
+  ["*dhcp_tag","",0,"DHCP Tag",True,"Should be used with --interface",0,"str"],
+  ["*dns_name","",0,"DNS Name",True,"Should be used with --interface",0,"str"],
+  ["*static_routes",[],0,"Static Routes",True,"Should be used with --interface",0,"list"],
+  ["*virt_bridge","",0,"Virt Bridge",True,"Should be used with --interface",0,"str"],
+  ["*ipv6_address","",0,"IPv6 Address",True,"Should be used with --interface",0,"str"],
+  ["*ipv6_secondaries",[],0,"IPv6 Secondaries",True,"Space delimited. Should be used with --interface",0,"list"],
+  ["*ipv6_mtu","",0,"IPv6 MTU",True,"Should be used with --interface",0,"str"],
+  ["*ipv6_static_routes",[],0,"IPv6 Static Routes",True,"Should be used with --interface",0,"list"],
+  ["*ipv6_default_gateway","",0,"IPv6 Default Gateway",True,"Should be used with --interface",0,"str"],
   ["mgmt_classes",[],0,"Management Classes",True,"For external config management",0,"list"],
   ["mgmt_parameters","<<inherit>>",0,"Management Parameters",True,"Parameters which will be handed to your management application (Must be valid YAML dictionary)", 0,"str"],
   [ "boot_files",{},'<<inherit>>',"TFTP Boot Files",True,"Files copied into tftpboot beyond the kernel/initrd",0,"list"],
@@ -93,6 +102,7 @@ FIELDS = [
   ["ldap_enabled",False,0,"LDAP Enabled",True,"(re)configure LDAP on this machine at next config update?",0,"bool"],
   ["ldap_type","SETTINGS:ldap_management_default_type",0,"LDAP Management Type",True,"Ex: authconfig",0,"str"],
   ["monit_enabled",False,0,"Monit Enabled",True,"(re)configure monit on this machine at next config update?",0,"bool"],
+  ["*cnames",[],0,"CNAMES",True,"Cannonical Name Records, should be used with --interface, In quotes, space delimited",0,"list"],
 ]
 
 class System(item.Item):
@@ -123,30 +133,54 @@ class System(item.Item):
                 raise CX(_("At least one interface needs to be defined."))
 
         return True
-        
+
+    def rename_interface(self,names):
+        """
+        Used to rename an interface.
+        """
+        (name,newname) = names
+        if not self.interfaces.has_key(name):
+            raise CX(_("Interface %s does not exist" % name))
+        if self.interfaces.has_key(newname):
+            raise CX(_("Interface %s already exists" % newname))
+        else:
+            self.interfaces[newname] = self.interfaces[name]
+            del self.interfaces[name]
+
+        return True
 
     def __get_interface(self,name):
 
-        if not self.interfaces.has_key(name):
+        if name == "" and len(self.interfaces.keys()) == 1:
+            name = self.interfaces.keys()[0]
+        elif name == "" and len(self.interfaces.keys()) > 1:
+            raise CX(_("Multiple interfaces defined. Please use --interface <interface_name>"))
+        elif not self.interfaces.has_key(name):
             self.interfaces[name] = {
-                "mac_address"    : "",
-                "mtu"            : "",
-                "ip_address"     : "",
-                "dhcp_tag"       : "",
-                "subnet"         : "",
-                "virt_bridge"    : "",
-                "static"         : False,
-                "bonding"        : "",
-                "bonding_master" : "",
-                "bonding_opts"   : "",
-                "management"     : False,
-                "dns_name"       : "",
-                "static_routes"  : [],
-                "ipv6_address"   : "",
-                "ipv6_secondaries"  : [],
-                "ipv6_mtu"       : "",
-                "ipv6_static_routes"  : [],
+                "mac_address"          : "",
+                "mtu"                  : "",
+                "ip_address"           : "",
+                "dhcp_tag"             : "",
+                "subnet"               : "", # deprecated
+                "netmask"              : "",
+                "if_gateway"           : "",
+                "virt_bridge"          : "",
+                "static"               : False,
+                "interface_type"       : "",
+                "interface_master"     : "",
+                "bonding"              : "", # deprecated
+                "bonding_master"       : "", # deprecated
+                "bonding_opts"         : "",
+                "bridge_opts"          : "",
+                "management"           : False,
+                "dns_name"             : "",
+                "static_routes"        : [],
+                "ipv6_address"         : "",
+                "ipv6_secondaries"     : [],
+                "ipv6_mtu"             : "",
+                "ipv6_static_routes"   : [],
                 "ipv6_default_gateway" : "",
+                "cnames"               : [],
             }
 
         return self.interfaces[name]
@@ -195,7 +229,7 @@ class System(item.Item):
         self.name = name 
 
         return True
-
+    
     def set_redhat_management_key(self,key):
         return utils.set_redhat_management_key(self,key)
 
@@ -212,6 +246,12 @@ class System(item.Item):
         self.server = server
         return True
 
+    def set_proxy(self,proxy):
+        if proxy is None or proxy == "":
+            proxy = "<<inherit>>"
+        self.proxy = proxy
+        return True
+
     def get_mac_address(self,interface):
         """
         Get the mac address, which may be implicit in the object name or explicit with --mac-address.
@@ -224,7 +264,7 @@ class System(item.Item):
             return intf["mac_address"].strip()
         else:
             return None
-
+        
     def get_ip_address(self,interface):
         """
         Get the IP address, which may be implicit in the object name or explict with --ip-address.
@@ -271,11 +311,17 @@ class System(item.Item):
            for x in matched:
                if x.name != self.name:
                    raise CX("dns-name duplicated: %s" % dns_name)
-
+               
 
         intf["dns_name"] = dns_name
         return True
  
+    def set_cnames(self,cnames,interface):
+        intf = self.__get_interface(interface)
+        data = utils.input_string_or_list(cnames)
+        intf["cnames"] = data        
+        return True
+    
     def set_static_routes(self,routes,interface):
         intf = self.__get_interface(interface)
         data = utils.input_string_or_list(routes)
@@ -288,6 +334,10 @@ class System(item.Item):
         self.hostname = hostname
         return True
 
+    def set_status(self,status):
+        self.status = status
+        return True
+
     def set_static(self,truthiness,interface):
         intf = self.__get_interface(interface)
         intf["static"] = utils.input_boolean(truthiness)
@@ -361,11 +411,18 @@ class System(item.Item):
         self.name_servers_search = data
         return True
 
-    def set_subnet(self,subnet,interface):
+    def set_netmask(self,netmask,interface):
         intf = self.__get_interface(interface)
-        intf["subnet"] = subnet
+        intf["netmask"] = netmask
         return True
     
+    def set_if_gateway(self,gateway,interface):
+        intf = self.__get_interface(interface)
+        if gateway == "" or utils.is_ip(gateway):
+            intf["if_gateway"] = gateway
+            return True
+        raise CX(_("invalid gateway: %s" % gateway))
+
     def set_virt_bridge(self,bridge,interface):
         if bridge == "":
             bridge = self.settings.default_virt_bridge
@@ -373,18 +430,25 @@ class System(item.Item):
         intf["virt_bridge"] = bridge
         return True
 
-    def set_bonding(self,bonding,interface):
-        if bonding not in ["master","slave","na",""] : 
-            raise CX(_("bonding value must be one of: master, slave, na"))
-        if bonding == "na":
-            bonding = ""
+    def set_interface_type(self,type,interface):
+        # master and slave are deprecated, and will
+        # be assumed to mean bonding slave/master
+        interface_types = ["bridge","bridge_slave","bond","bond_slave","bonded_bridge_slave","master","slave","na",""]
+        if type not in interface_types:
+            raise CX(_("interface type value must be one of: %s or blank" % interface_types.join(",")))
+        if type == "na":
+            type = ""
+        elif type == "master":
+            type = "bond"
+        elif type == "slave":
+            type = "bond_slave"
         intf = self.__get_interface(interface)
-        intf["bonding"] = bonding
+        intf["interface_type"] = type
         return True
 
-    def set_bonding_master(self,bonding_master,interface):
+    def set_interface_master(self,interface_master,interface):
         intf = self.__get_interface(interface)
-        intf["bonding_master"] = bonding_master
+        intf["interface_master"] = interface_master
         return True
 
     def set_bonding_opts(self,bonding_opts,interface):
@@ -392,6 +456,11 @@ class System(item.Item):
         intf["bonding_opts"] = bonding_opts
         return True
 
+    def set_bridge_opts(self,bridge_opts,interface):
+        intf = self.__get_interface(interface)
+        intf["bridge_opts"] = bridge_opts
+        return True
+
     def set_ipv6_autoconfiguration(self,truthiness):
         self.ipv6_autoconfiguration = utils.input_boolean(truthiness)
         return True
@@ -449,6 +518,13 @@ class System(item.Item):
         intf["mtu"] = mtu
         return True
 
+    def set_enable_gpxe(self,enable_gpxe):
+        """
+        Sets whether or not the system will use gPXE for booting.
+        """
+        self.enable_gpxe = utils.input_boolean(enable_gpxe)
+        return True
+
     def set_profile(self,profile_name):
         """
         Set the system to use a certain named profile.  The profile
@@ -507,10 +583,16 @@ class System(item.Item):
 
     def set_virt_file_size(self,num):
         return utils.set_virt_file_size(self,num)
+
+    def set_virt_disk_driver(self,driver):
+        return utils.set_virt_disk_driver(self,driver)
  
     def set_virt_auto_boot(self,num):
         return utils.set_virt_auto_boot(self,num)
 
+    def set_virt_pxe_boot(self,num):
+        return utils.set_virt_pxe_boot(self,num)
+
     def set_virt_ram(self,num):
         return utils.set_virt_ram(self,num)
 
@@ -552,7 +634,8 @@ class System(item.Item):
         if kickstart is None or kickstart in [ "", "delete", "<<inherit>>" ]:
             self.kickstart = "<<inherit>>"
             return True
-        if utils.find_kickstart(kickstart):
+        kickstart = utils.find_kickstart(kickstart)
+        if kickstart:
             self.kickstart = kickstart
             return True
         raise CX(_("kickstart not found: %s" % kickstart))
@@ -565,7 +648,7 @@ class System(item.Item):
         choices = utils.get_power_types()
         choices.sort()
         if power_type not in choices:
-            raise CX("power type must be one of: %s" % ",".join(choices))
+            raise CX("power management type must be one of: %s" % ",".join(choices))
         self.power_type = power_type
         return True
 
@@ -610,11 +693,16 @@ class System(item.Item):
             if field == "dnsname"             : self.set_dns_name(value, interface)
             if field == "static"              : self.set_static(value, interface)
             if field == "dhcptag"             : self.set_dhcp_tag(value, interface)
-            if field == "subnet"              : self.set_subnet(value, interface)
+            if field == "netmask"             : self.set_netmask(value, interface)
+            if field == "subnet"              : self.set_netmask(value, interface)
+            if field == "ifgateway"           : self.set_if_gateway(value, interface)
             if field == "virtbridge"          : self.set_virt_bridge(value, interface)
-            if field == "bonding"             : self.set_bonding(value, interface)
-            if field == "bondingmaster"       : self.set_bonding_master(value, interface)
+            if field == "interfacetype"       : self.set_interface_type(value, interface)
+            if field == "interfacemaster"     : self.set_interface_master(value, interface)
+            if field == "bonding"             : self.set_interface_type(value, interface)   # deprecated
+            if field == "bondingmaster"       : self.set_interface_master(value, interface) # deprecated
             if field == "bondingopts"         : self.set_bonding_opts(value, interface)
+            if field == "bridgeopts"          : self.set_bridge_opts(value, interface)
             if field == "management"          : self.set_management(value, interface)
             if field == "staticroutes"        : self.set_static_routes(value, interface)
             if field == "ipv6address"         : self.set_ipv6_address(value, interface)
@@ -622,6 +710,7 @@ class System(item.Item):
             if field == "ipv6mtu"             : self.set_ipv6_mtu(value, interface)
             if field == "ipv6staticroutes"    : self.set_ipv6_static_routes(value, interface)
             if field == "ipv6defaultgateway"  : self.set_ipv6_default_gateway(value, interface)
+            if field == "cnames"              : self.set_cnames(value, interface)
 
         return True
 
