@@ -1,8 +1,8 @@
 """
 This is some of the code behind 'cobbler sync'.
 
-Copyright 2006-2009, Red Hat, Inc
-Michael DeHaan <mdehaan@redhat.com>
+Copyright 2006-2009, Red Hat, Inc and Others
+Michael DeHaan <michael.dehaan AT gmail>
 John Eckersberg <jeckersb@redhat.com>
 
 This program is free software; you can redistribute it and/or modify
@@ -24,14 +24,9 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 import os
 import os.path
 import shutil
-import time
-import sys
 import glob
 import traceback
-import errno
-from utils import popen2
-from shlex import shlex
-
+import string
 
 import utils
 from cexceptions import *
@@ -155,6 +150,10 @@ class ImportRedhatManager:
         # FIXME : search below self.path for isolinux configurations or known directories from TRY_LIST
         os.path.walk(self.path, self.distro_adder, distros_added)
 
+        if len(distros_added) == 0:
+            self.logger.warning("No distros imported, bailing out")
+            return False
+
         # find out if we can auto-create any repository records from the install tree
 
         if self.network_root is None:
@@ -198,39 +197,6 @@ class ImportRedhatManager:
                 data2.append(x)
         return data2
 
-    def get_tree_location(self, distro):
-        """
-        Once a distribution is identified, find the part of the distribution
-        that has the URL in it that we want to use for kickstarting the
-        distribution, and create a ksmeta variable $tree that contains this.
-        """
-
-        base = self.get_rootdir()
-
-        if self.network_root is None:
-            dest_link = os.path.join(self.settings.webdir, "links", distro.name)
-            # create the links directory only if we are mirroring because with
-            # SELinux Apache can't symlink to NFS (without some doing)
-            if not os.path.exists(dest_link):
-                try:
-                    os.symlink(base, dest_link)
-                except:
-                    # this shouldn't happen but I've seen it ... debug ...
-                    self.logger.warning("symlink creation failed: %(base)s, %(dest)s") % { "base" : base, "dest" : dest_link }
-            # how we set the tree depends on whether an explicit network_root was specified
-            tree = "http://@@http_server@@/cblr/links/%s" % (distro.name)
-            self.set_install_tree(distro, tree)
-        else:
-            # where we assign the kickstart source is relative to our current directory
-            # and the input start directory in the crawl.  We find the path segments
-            # between and tack them on the network source path to find the explicit
-            # network path to the distro that Anaconda can digest.
-            tail = utils.path_tail(self.path, base)
-            tree = self.network_root[:-1] + tail
-            self.set_install_tree(distro, tree)
-
-        return
-
     def repo_finder(self, distros_added):
         """
         This routine looks through all distributions and tries to find
@@ -368,8 +334,6 @@ class ImportRedhatManager:
         pae_initrd = None
         pae_kernel = None
 
-        print "in distro_adder(), directory is %s" % dirname
-
         for x in fnames:
             adtls = []
 
