@@ -21,10 +21,12 @@ import os.path, traceback, errno
 import re
 import clogger
 import pxegen
+import shutil
+import glob
 
 import utils
 from cexceptions import *
-import templar 
+import templar
 
 from utils import _
 
@@ -69,31 +71,33 @@ class InTftpdManager:
         target      = utils.blender(self.config.api, False, distro)
 
         # Create metadata for the templar function
-        # Right now, just using img_path, but adding more
+        # Right now, just using local_img_path, but adding more
         # cobbler variables here would probably be good
         metadata = {}
-        metadata["img_path"] = os.path.join(
-                                    utils.tftpboot_location(),
-                                    "images",distro.name)
+        metadata["local_img_path"] = os.path.join(utils.tftpboot_location(),"images",distro.name)
 	# Create the templar instance.  Used to template the target directory
-	templater = templar.Templar()
+	templater = templar.Templar(self.config)
 
         # Loop through the hash of boot files,
         # executing a cp for each one
         for file in target["boot_files"].keys():
-            file_dst = templater.render(file,metadata,None)
+            rendered_file = templater.render(file,metadata,None)
             try:
-                shutil.copyfile(target["boot_files"][file], file_dst)
-                self.config.api.log("copied file %s to %s for %s" % (
-                        target["boot_files"][file],
-                        file_dst,
-                        distro.name))
+                for f in glob.glob(target["boot_files"][file]):
+                    if f == target["boot_files"][file]:
+                        # this wasn't really a glob, so just copy it as is
+                        filedst = rendered_file
+                    else:
+                        # this was a glob, so figure out what the destination
+                        # file path/name should be
+                        tgt_path,tgt_file=os.path.split(f)
+                        rnd_path,rnd_file=os.path.split(rendered_file)
+                        filedst = os.path.join(rnd_path,tgt_file)
+                    if not os.path.isfile(filedst):
+                        shutil.copyfile(f, filedst)
+                    self.config.api.log("copied file %s to %s for %s" % (f,filedst,distro.name))
             except:
-                self.logger.error("failed to copy file %s to %s for %s" % (
-                        target["boot_files"][file],
-                        file_dst,
-                    distro.name))
-                # Continue on to sync what you can
+                self.logger.error("failed to copy file %s to %s for %s" % (f,filedst,distro.name))
 
         return 0
 
@@ -125,7 +129,7 @@ class InTftpdManager:
         metadata = {
             "user"      : "root",
             "binary"    : "/usr/sbin/in.tftpd",
-            "args"      : "-v -s %s" % self.bootloc
+            "args"      : "%s" % self.bootloc
         }
         self.logger.info("generating %s" % self.settings_file)
         self.templar.render(template_data, metadata, self.settings_file, None)
