@@ -1,8 +1,8 @@
 """
 This is some of the code behind 'cobbler sync'.
 
-Copyright 2006-2009, Red Hat, Inc
-Michael DeHaan <mdehaan@redhat.com>
+Copyright 2006-2009, Red Hat, Inc and Others
+Michael DeHaan <michael.dehaan AT gmail>
 John Eckersberg <jeckersb@redhat.com>
 
 This program is free software; you can redistribute it and/or modify
@@ -21,17 +21,10 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 02110-1301  USA
 """
 
-import os
-import os.path
-import shutil
 import time
-import sys
 import glob
 import traceback
 import errno
-from utils import popen2
-from shlex import shlex
-
 
 import utils
 from cexceptions import *
@@ -117,7 +110,7 @@ class IscManager:
                 interface["gateway"] = system.gateway
 
                 mac  = interface["mac_address"]
-                if interface["interface_type"] in ("slave","bond_slave","bridge_slave"):
+                if interface["interface_type"] in ("slave","bond_slave","bridge_slave","bonded_bridge_slave"):
                     if interface["interface_master"] not in system.interfaces:
                         # Can't write DHCP entry; master interface does not
                         # exist
@@ -158,6 +151,11 @@ class IscManager:
                 interface["next_server"] = blended_system["server"]
                 interface["netboot_enabled"] = blended_system["netboot_enabled"]
                 interface["hostname"] = blended_system["hostname"]
+                interface["owner"] = blended_system["name"]
+                interface["enable_gpxe"] = blended_system["enable_gpxe"]
+
+                if not interface["netboot_enabled"] and interface['static']:
+                    continue
 
                 interface["filename"] = "/pxelinux.0"
                 # can't use pxelinux.0 anymore
@@ -182,7 +180,7 @@ class IscManager:
         # we are now done with the looping through each interface of each system
         metadata = {
            "date"           : time.asctime(time.gmtime()),
-           "cobbler_server" : self.settings.server,
+           "cobbler_server" : "%s:%s" % (self.settings.server,self.settings.http_port),
            "next_server"    : self.settings.next_server,
            "elilo"          : elilo,
            "yaboot"         : yaboot,
