@@ -2,8 +2,8 @@
 Builds out filesystem trees/data based on the object tree.
 This is the code behind 'cobbler sync'.
 
-Copyright 2006-2009, Red Hat, Inc
-Michael DeHaan <mdehaan@redhat.com>
+Copyright 2006-2009, Red Hat, Inc and Others
+Michael DeHaan <michael.dehaan AT gmail>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -117,6 +117,10 @@ class PXEGen:
             utils.copyfile_pattern('/usr/share/syslinux/memdisk', dst,
                     require_match=False, api=self.api, cache=False, logger=self.logger)
 
+        # Copy gPXE/iPXE bootloader if it exists
+        utils.copyfile_pattern('/usr/share/*pxe/undionly.kpxe', dst,
+                require_match=False, api=self.api, cache=False, logger=self.logger)
+
         # Copy grub EFI bootloaders if possible:
         utils.copyfile_pattern('/var/lib/cobbler/loaders/grub*.efi', grub_dst,
                 require_match=False, api=self.api, cache=False, logger=self.logger)
@@ -226,6 +230,8 @@ class PXEGen:
                 utils.rmfile(f2)
             return
 
+        pxe_metadata = {'pxe_menu_items': self.get_menu_items()['pxe'] }
+ 
         # generate one record for each described NIC ..
  
         for (name,interface) in system.interfaces.iteritems():
@@ -247,7 +253,7 @@ class PXEGen:
 
             # for tftp only ...
             grub_path = None
-            if working_arch in [ "i386", "x86", "x86_64", "standard"]:
+            if working_arch in [ "i386", "x86", "x86_64", "arm", "standard"]:
                 # pxelinux wants a file named $name under pxelinux.cfg
                 f2 = os.path.join(self.bootloc, "pxelinux.cfg", f1)
 
@@ -278,12 +284,12 @@ class PXEGen:
 
             if system.is_management_supported():
                 if not image_based:
-                    self.write_pxe_file(f2, system, profile, distro, working_arch)
+                    self.write_pxe_file(f2, system, profile, distro, working_arch, metadata=pxe_metadata)
                     if grub_path:
                         self.write_pxe_file(grub_path, system, profile, distro, 
                                 working_arch, format="grub")
                 else:
-                    self.write_pxe_file(f2, system, None, None, working_arch, image=profile)
+                    self.write_pxe_file(f2, system, None, None, working_arch, image=profile, metadata=pxe_metadata)
             else:
                 # ensure the file doesn't exist
                 utils.rmfile(f2)
@@ -335,18 +341,10 @@ class PXEGen:
 
         listfile.close()
 
-    def make_actual_pxe_menu(self):
+    def get_menu_items(self):
         """
-        Generates both pxe and grub boot menus.
+        Generates menu items for pxe and grub
         """
-        # only do this if there is NOT a system named default.
-        default = self.systems.find(name="default")
-
-        if default is None:
-            timeout_action = "local"
-        else:
-            timeout_action = default.profile
-
         # sort the profiles
         profile_list = [profile for profile in self.profiles]
         def sort_name(a,b):
@@ -395,16 +393,32 @@ class PXEGen:
 
         # if we have any memtest files in images, make entries for them
         # after we list the profiles
-        memtests = glob.glob(self.bootloc + "/memtest*")
+        memtests = glob.glob(self.bootloc + "/images/memtest*")
         if len(memtests) > 0:
             pxe_menu_items = pxe_menu_items + "\n\n"
-            for memtest in glob.glob(self.bootloc + '/memtest*'):
+            for memtest in glob.glob(self.bootloc + '/images/memtest*'):
                 base = os.path.basename(memtest)
                 contents = self.write_memtest_pxe("/%s" % base)
                 pxe_menu_items = pxe_menu_items + contents + "\n"
               
+        return {'pxe' : pxe_menu_items, 'grub' : grub_menu_items}
+
+    def make_actual_pxe_menu(self):
+        """
+        Generates both pxe and grub boot menus.
+        """
+        # only do this if there is NOT a system named default.
+        default = self.systems.find(name="default")
+
+        if default is None:
+            timeout_action = "local"
+        else:
+            timeout_action = default.profile
+
+        menu_items = self.get_menu_items()
+              
         # Write the PXE menu:
-        metadata = { "pxe_menu_items" : pxe_menu_items, "pxe_timeout_profile" : timeout_action}
+        metadata = { "pxe_menu_items" : menu_items['pxe'], "pxe_timeout_profile" : timeout_action}
         outfile = os.path.join(self.bootloc, "pxelinux.cfg", "default")
         template_src = open(os.path.join(self.settings.pxe_template_dir,"pxedefault.template"))
         template_data = template_src.read()
@@ -412,7 +426,7 @@ class PXEGen:
         template_src.close()
 
         # Write the grub menu:
-        metadata = { "grub_menu_items" : grub_menu_items }
+        metadata = { "grub_menu_items" : menu_items['grub'] }
         outfile = os.path.join(self.bootloc, "grub", "efidefault")
         template_src = open(os.path.join(self.settings.pxe_template_dir, "efidefault.template"))
         template_data = template_src.read()
@@ -437,7 +451,7 @@ class PXEGen:
         # store variables for templating
         metadata["menu_label"] = "MENU LABEL %s" % os.path.basename(filename)
         metadata["profile_name"] = os.path.basename(filename)
-        metadata["kernel_path"] = "/%s" % os.path.basename(filename)
+        metadata["kernel_path"] = "/images/%s" % os.path.basename(filename)
         metadata["initrd_path"] = ""
         metadata["append_line"] = ""
 
@@ -476,6 +490,11 @@ class PXEGen:
 
         if metadata is None:
             metadata = {}
+
+        (rval,settings) = utils.input_string_or_hash(self.settings.to_datastruct())
+        if rval:
+            for key in settings.keys():
+                metadata[key] = settings[key]
         # ---
         # just some random variables
         template = None
@@ -535,7 +554,9 @@ class PXEGen:
                         template = os.path.join(self.settings.pxe_template_dir,"pxesystem_ia64.template")
                     elif arch.startswith("ppc"):
                         template = os.path.join(self.settings.pxe_template_dir,"pxesystem_ppc.template")
-                    elif distro.os_version.startswith("esxi"):
+                    elif arch.startswith("arm"):
+                        template = os.path.join(self.settings.pxe_template_dir,"pxesystem_arm.template")
+                    elif distro and distro.os_version.startswith("esxi"):
                         # ESXi uses a very different pxe method, using more files than
                         # a standard kickstart and different options - so giving it a dedicated
                         # PXE template makes more sense than shoe-horning it into the existing
@@ -572,6 +593,8 @@ class PXEGen:
             # not a system record, so this is a profile record or an image
             if arch.startswith("s390"):
                 template = os.path.join(self.settings.pxe_template_dir,"pxeprofile_s390x.template")
+            if arch.startswith("arm"):
+                template = os.path.join(self.settings.pxe_template_dir,"pxeprofile_arm.template")
             elif format == "grub":
                 template = os.path.join(self.settings.pxe_template_dir,"grubprofile.template")
             elif distro and distro.os_version.startswith("esxi"):
@@ -591,9 +614,9 @@ class PXEGen:
                 image, arch, kickstart_path)
         metadata["kernel_options"] = kernel_options
 
-        if distro.os_version.startswith("esxi") and filename is not None:
+        if distro and distro.os_version.startswith("esxi") and filename is not None:
             append_line = "BOOTIF=%s" % (os.path.basename(filename))
-        elif metadata.has_key("initrd_path") and (not arch or arch not in ["ia64", "ppc", "ppc64"]):
+        elif metadata.has_key("initrd_path") and (not arch or arch not in ["ia64", "ppc", "ppc64", "arm"]):
             append_line = "append initrd=%s" % (metadata["initrd_path"])
         else:
             append_line = "append "
@@ -643,8 +666,18 @@ class PXEGen:
         Builds the full kernel options line.
         """
 
+        management_interface = None
         if system is not None:
             blended = utils.blender(self.api, False, system)
+            # find the first management interface
+            try:
+                for intf in system.interfaces.keys():
+                    if system.interfaces[intf]["management"]:
+                        management_interface = intf
+                        break
+            except:
+                # just skip this then
+                pass
         elif profile is not None:
             blended = utils.blender(self.api, False, profile)
         else:
@@ -657,10 +690,6 @@ class PXEGen:
             append_line = ",%s" % kopts.pop("initrd")
         hkopts = utils.hash_to_string(kopts)
         append_line = "%s %s" % (append_line, hkopts)
-        # FIXME - the append_line length limit is architecture specific
-        # TODO: why is this checked here, before we finish adding everything?
-        if len(append_line) >= 255:
-            self.logger.warning("warning: kernel option length exceeds 255")
 
         # kickstart path rewriting (get URLs for local files)
         if kickstart_path is not None and kickstart_path != "":
@@ -678,10 +707,17 @@ class PXEGen:
 
             if distro.breed is None or distro.breed == "redhat":
                 append_line = "%s ks=%s" % (append_line, kickstart_path)
+                gpxe = blended["enable_gpxe"]
+                if gpxe:
+                    append_line = append_line.replace('ksdevice=bootif','ksdevice=${net0/mac}')
             elif distro.breed == "suse":
                 append_line = "%s autoyast=%s" % (append_line, kickstart_path)
             elif distro.breed == "debian" or distro.breed == "ubuntu":
-                append_line = "%s auto url=%s" % (append_line, kickstart_path)
+                append_line = "%s auto-install/enable=true priority=critical url=%s" % (append_line, kickstart_path)
+                if management_interface:
+                    append_line += " netcfg/choose_interface=%s" % management_interface
+            elif distro.breed == "freebsd":
+                append_line = "%s ks=%s" % (append_line, kickstart_path)
 
                 # rework kernel options for debian distros
                 translations = { 'ksdevice':"interface" , 'lang':"locale" }
@@ -712,7 +748,7 @@ class PXEGen:
             # using LVM.
             domain = "local.lan"
             if system is not None:
-                if system.hostname is not None:
+                if system.hostname is not None and system.hostname != "":
                     # If this is a FQDN, grab the first bit
                     hostname = system.hostname.split(".")[0]
                     _domain = system.hostname.split(".")[1:]
@@ -721,7 +757,11 @@ class PXEGen:
                 else:
                     hostname = system.name
             else:
-                hostname = profile.name
+                # ubuntu at the very least does not like having underscores
+                # in the hostname. 
+                # FIXME: Really this should remove all characters that are 
+                # forbidden in hostnames
+                hostname = profile.name.replace("_","")
 
             # At least for debian deployments configured for DHCP networking
             # this values are not used, but specifying here avoids questions
@@ -732,6 +772,21 @@ class PXEGen:
             # the existence of "stable" in the dists directory
             append_line = "%s suite=%s" % (append_line, distro.os_version)
 
+        # append necessary kernel args for arm architectures
+        if arch is not None and arch.startswith("arm"):
+            append_line = "%s fixrtc vram=48M omapfb.vram=0:24M" % append_line
+
+        # do variable substitution on the append line
+        # FIXME: should we just promote all of the ksmeta 
+        #        variables instead of just the tree?
+        if blended.has_key("ks_meta") and blended["ks_meta"].has_key("tree"):
+            blended["tree"] = blended["ks_meta"]["tree"]
+        append_line = self.templar.render(append_line,utils.flatten(blended),None)
+
+        # FIXME - the append_line length limit is architecture specific
+        if len(append_line) >= 255:
+            self.logger.warning("warning: kernel option length exceeds 255")
+
         return append_line
 
     def write_templates(self,obj,write_file=False,path=None):
@@ -746,6 +801,7 @@ class PXEGen:
         names (after variable substitution is done) and the
         data in the file.
         """
+        self.logger.info("Writing template files for %s" % obj.name)
 
         results = {}
 
@@ -775,6 +831,12 @@ class PXEGen:
         if not success:
             return results
 
+        # FIXME: img_path and local_img_path should probably be moved
+        #        up into the blender function to ensure they're consistently
+        #        available to templates across the board
+        if blended["distro_name"]:
+            blended['img_path'] = os.path.join("/images",blended["distro_name"])
+            blended['local_img_path'] = os.path.join(utils.tftpboot_location(),"images",blended["distro_name"])
 
         for template in templates.keys():
             dest = templates[template]
@@ -784,22 +846,23 @@ class PXEGen:
             # Run the source and destination files through 
             # templar first to allow for variables in the path 
             template = self.templar.render(template, blended, None).strip()
-            dest     = self.templar.render(dest, blended, None).strip()
+            dest     = os.path.normpath(self.templar.render(dest, blended, None).strip())
             # Get the path for the destination output
-            dest_dir = os.path.dirname(dest)
+            dest_dir = os.path.normpath(os.path.dirname(dest))
 
             # If we're looking for a single template, skip if this ones
             # destination is not it.
             if not path is None and path != dest:
                continue
 
-            # If we are writing output to a file, force all templated 
-            # configs into the rendered directory to ensure that a user 
-            # granted cobbler privileges via sudo can't overwrite 
+            # If we are writing output to a file, we allow files tobe 
+            # written into the tftpboot directory, otherwise force all 
+            # templated configs into the rendered directory to ensure that 
+            # a user granted cobbler privileges via sudo can't overwrite 
             # arbitrary system files (This also makes cleanup easier).
-            if os.path.isabs(dest_dir):
-               if write_file:
-                   raise CX(" warning: template destination (%s) is an absolute path, skipping." % dest_dir)
+            if os.path.isabs(dest_dir) and write_file:
+                if dest_dir.find(utils.tftpboot_location()) != 0:
+                   raise CX(" warning: template destination (%s) is outside %s, skipping." % (dest_dir,utils.tftpboot_location()))
                    continue
             else:
                 dest_dir = os.path.join(self.settings.webdir, "rendered", dest_dir)
@@ -839,3 +902,167 @@ class PXEGen:
 
         return results
 
+    def generate_gpxe(self,what,name):
+       if what.lower() not in ("profile","system"):
+           return "# gpxe is only valid for profiles and systems"
+
+       distro = None
+       if what == "profile":
+           obj = self.api.find_profile(name=name)
+           distro = obj.get_conceptual_parent()
+       else:
+           obj = self.api.find_system(name=name)
+           distro = obj.get_conceptual_parent().get_conceptual_parent()
+           netboot_enabled = obj.netboot_enabled
+
+       # For multi-arch distros, the distro name in ks_mirror
+       # may not contain the arch string, so we need to figure out
+       # the path based on where the kernel is stored. We do this 
+       # because some distros base future downloads on the initial
+       # URL passed in, so all of the files need to be at this location
+       # (which is why we can't use the images link, which just contains
+       # the kernel and initrd).
+       ks_mirror_name = string.join(distro.kernel.split('/')[-2:-1],'')
+
+       blended = utils.blender(self.api, False, obj)
+
+       ksmeta = blended.get("ks_meta",{})
+       try:
+           del blended["ks_meta"]
+       except:
+           pass
+       blended.update(ksmeta) # make available at top level
+
+       blended['distro'] = distro.name
+       blended['ks_mirror_name'] = ks_mirror_name
+       blended['kernel_name'] = os.path.basename(distro.kernel)
+       blended['initrd_name'] = os.path.basename(distro.initrd)
+
+       if what == "profile":
+           blended['append_line'] = self.build_kernel_options(None,obj,distro,None,None,blended['kickstart'])
+       else:
+           blended['append_line'] = self.build_kernel_options(obj,None,distro,None,None,blended['kickstart'])
+
+       template = None
+       if distro.breed in ['redhat','debian','ubuntu','suse']:
+           # all of these use a standard kernel/initrd setup so
+           # they all use the same gPXE template
+           template = os.path.join(self.settings.pxe_template_dir,"gpxe_%s_linux.template" % what.lower())
+       elif distro.breed == 'vmware':
+           if distro.os_version == 'esx4':
+               # older ESX is pretty much RHEL, so it uses the standard kernel/initrd setup
+               template = os.path.join(self.settings.pxe_template_dir,"gpxe_%s_linux.template" % what.lower())
+           elif distro.os_version == 'esxi4':
+               template = os.path.join(self.settings.pxe_template_dir,"gpxe_%s_esxi4.template" % what.lower())
+           elif distro.os_version.startswith('esxi5'):
+               template = os.path.join(self.settings.pxe_template_dir,"gpxe_%s_esxi5.template" % what.lower())
+       elif distro.breed == 'freebsd':
+           template = os.path.join(self.settings.pxe_template_dir,"gpxe_%s_freebsd.template" % what.lower())
+
+       if what == "system":
+           if not netboot_enabled:
+               template = os.path.join(self.settings.pxe_template_dir,"gpxe_%s_local.template" % what.lower())
+
+
+       if not template:
+           return "# unsupported breed/os version"
+
+       if not os.path.exists(template):
+           return "# gpxe template not found for the %s named %s (filename=%s)" % (what,name,template)
+
+       template_fh = open(template)
+       template_data = template_fh.read()
+       template_fh.close()
+
+       return self.templar.render(template_data, blended, None)
+
+    def generate_bootcfg(self,what,name):
+       if what.lower() not in ("profile","system"):
+           return "# bootcfg is only valid for profiles and systems"
+
+       distro = None
+       if what == "profile":
+           obj = self.api.find_profile(name=name)
+           distro = obj.get_conceptual_parent()
+       else:
+           obj = self.api.find_system(name=name)
+           distro = obj.get_conceptual_parent().get_conceptual_parent()
+
+       # For multi-arch distros, the distro name in ks_mirror
+       # may not contain the arch string, so we need to figure out
+       # the path based on where the kernel is stored. We do this
+       # because some distros base future downloads on the initial
+       # URL passed in, so all of the files need to be at this location
+       # (which is why we can't use the images link, which just contains
+       # the kernel and initrd).
+       ks_mirror_name = string.join(distro.kernel.split('/')[-2:-1],'')
+
+       blended = utils.blender(self.api, False, obj)
+
+       ksmeta = blended.get("ks_meta",{})
+       try:
+           del blended["ks_meta"]
+       except:
+           pass
+       blended.update(ksmeta) # make available at top level
+
+       blended['distro'] = ks_mirror_name
+
+       # FIXME: img_path should probably be moved up into the 
+       #        blender function to ensure they're consistently
+       #        available to templates across the board
+       if obj.enable_gpxe:
+           blended['img_path'] = 'http://%s:%s/cobbler/links/%s' % (self.settings.server,self.settings.http_port,distro.name)
+       else:
+           blended['img_path'] = os.path.join("/images",distro.name)
+
+       template = os.path.join(self.settings.pxe_template_dir,"bootcfg_%s_%s.template" % (what.lower(),distro.os_version))
+       if not os.path.exists(template):
+           return "# boot.cfg template not found for the %s named %s (filename=%s)" % (what,name,template)
+
+       template_fh = open(template)
+       template_data = template_fh.read()
+       template_fh.close()
+
+       return self.templar.render(template_data, blended, None)
+
+    def generate_script(self,what,objname,script_name):
+       if what == "profile":
+           obj = self.api.find_profile(name=objname)
+       else:
+           obj = self.api.find_system(name=objname)
+
+       if not obj:
+           return "# %s named %s not found" % (what,objname)
+
+       distro = obj.get_conceptual_parent()
+       while distro.get_conceptual_parent():
+           distro = distro.get_conceptual_parent()
+
+       blended = utils.blender(self.api, False, obj)
+
+       ksmeta = blended.get("ks_meta",{})
+       try:
+           del blended["ks_meta"]
+       except:
+           pass
+       blended.update(ksmeta) # make available at top level
+
+       # FIXME: img_path should probably be moved up into the
+       #        blender function to ensure they're consistently
+       #        available to templates across the board
+       if obj.enable_gpxe:
+           blended['img_path'] = 'http://%s:%s/cobbler/links/%s' % (self.settings.server,self.settings.http_port,distro.name)
+       else:
+           blended['img_path'] = os.path.join("/images",distro.name)
+
+       template = os.path.normpath(os.path.join("/var/lib/cobbler/scripts",script_name))
+       if not os.path.exists(template):
+           return "# script template %s not found" % script_name
+
+       template_fh = open(template)
+       template_data = template_fh.read()
+       template_fh.close()
+
+       return self.templar.render(template_data, blended, None, obj)
+
