@@ -1,8 +1,8 @@
 """
 Misc heavy lifting functions for cobbler
 
-Copyright 2006-2009, Red Hat, Inc
-Michael DeHaan <mdehaan@redhat.com>
+Copyright 2006-2009, Red Hat, Inc and Others
+Michael DeHaan <michael.dehaan AT gmail>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -107,9 +107,12 @@ def _(foo):
    return foo
 
 MODULE_CACHE = {}
+SIGNATURE_CACHE = {}
 
 _re_kernel = re.compile(r'(vmlinu[xz]|kernel.img)')
 _re_initrd = re.compile(r'(initrd(.*).img|ramdisk.image.gz)')
+_re_is_mac = re.compile(':'.join(('[0-9A-Fa-f][0-9A-Fa-f]',)*6) + '$')
+_re_is_ibmac = re.compile(':'.join(('[0-9A-Fa-f][0-9A-Fa-f]',)*20) + '$')
 
 # all logging from utils.die goes to the main log even if there
 # is another log.
@@ -260,12 +263,9 @@ def is_mac(strdata):
     """
     Return whether the argument is a mac address.
     """
-    # needs testcase
     if strdata is None:
         return False
-    if re.search(r'[A-F0-9]{2}:[A-F0-9]{2}:[A-F0-9]{2}:[A-F0-9]{2}:[A-F:0-9]{2}:[A-F:0-9]{2}',strdata, re.IGNORECASE):
-        return True
-    return False
+    return bool(_re_is_mac.match(strdata) or _re_is_ibmac.match(strdata))
 
 def get_random_mac(api_handle,virt_type="xenpv"):
     """
@@ -279,7 +279,7 @@ def get_random_mac(api_handle,virt_type="xenpv"):
             random.randint(0x00, 0xff),
             random.randint(0x00, 0xff)
         ]
-    elif virt_type.startswith("xen") or virt_type.startswith("qemu"):
+    elif virt_type.startswith("xen") or virt_type.startswith("qemu") or virt_type.startswith("kvm"):
         mac = [ 0x00, 0x16, 0x3e,
             random.randint(0x00, 0x7f),
             random.randint(0x00, 0xff),
@@ -432,12 +432,15 @@ def find_kickstart(url):
     """
     if url is None:
         return None
-    x = url.lower()
+    x = url.lstrip()
     for y in ["http://", "nfs://", "ftp://", "/"]:
+       # make sure we get a lower-case protocol without 
+       # affecting the rest of the string
+       x = re.sub(r"(?i)%s" % y, y, x, count=1)
        if x.startswith(y):
-           if x.startswith("/") and not os.path.isfile(url):
+           if x.startswith("/") and not os.path.isfile(x):
                return None
-           return url
+           return x
     return None
 
 
@@ -468,7 +471,8 @@ def read_file_contents(file_location, logger=None, fetch_if_remote=False):
             f.close()
             return data
         except:
-            log_exc(logger)
+            if logger:
+                log_exc(logger)
             raise
 
     # Remote files:
@@ -517,7 +521,6 @@ def input_string_or_list(options):
     """
     Accepts a delimited list of stuff or a list, but always returns a list.
     """
-    delim = None
     if options == "<<inherit>>":
        return "<<inherit>>"
     if options is None or options == "" or options == "delete":
@@ -525,7 +528,7 @@ def input_string_or_list(options):
     elif isinstance(options,list):
        return options
     elif isinstance(options,basestring):
-       tokens = options.split(delim)
+       tokens = shlex.split(options)
        return tokens
     else:
        raise CX(_("invalid input type"))
@@ -587,6 +590,16 @@ def input_boolean(value):
     else:
        return False
 
+def update_settings_file(data):
+    if 1:#try:
+        #clogger.Logger().debug("in update_settings_file(): value is: %s" % str(value))
+        settings_file = file("/etc/cobbler/settings","w")
+        yaml.safe_dump(data,settings_file)
+        settings_file.close()
+        return True
+    #except:
+    #    return False
+
 def grab_tree(api_handle, obj):
     """
     Climb the tree and get every node.
@@ -658,6 +671,19 @@ def blender(api_handle,remove_hashes, root_obj):
                     if not results.has_key(key):
                         results[key] = interface[key]
 
+    # if the root object is a profile or system, add in all 
+    # repo data for repos that belong to the object chain
+    if root_obj.COLLECTION_TYPE in ("profile","system"):
+        repo_data = []
+        for r in results["repos"]:
+            repo = api_handle.find_repo(name=r)
+            if repo:
+                repo_data.append(repo.to_datastruct())
+        # FIXME: sort the repos in the array based on the 
+        #        repo priority field so that lower priority
+        #        repos come first in the array
+        results["repo_data"] = repo_data
+
     http_port = results.get("http_port",80)
     if http_port not in (80, "80"):
        results["http_server"] = "%s:%s" % (results["server"] , http_port)
@@ -791,8 +817,13 @@ def __consolidate(node,results):
              results[field].extend(data_item)
              results[field] = uniquify(results[field])
           else:
-             # just override scalars
-             results[field] = data_item
+             # distro field gets special handling, since we don't
+             # want to overwrite it ever.
+             # FIXME: should the parent's field too? It will be over-
+             #        written if there are multiple sub-profiles in
+             #        the chain of inheritance
+             if field != "distro":
+                results[field] = data_item
        else:
           results[field] = data_item
 
@@ -1024,32 +1055,30 @@ def os_release():
       return ("unknown",0)
 
 def tftpboot_location():
-
-    # if possible, read from TFTP config file to get the location
-    if os.path.exists("/etc/xinetd.d/tftp"):
-        fd = open("/etc/xinetd.d/tftp")
-        lines = fd.read().split("\n")
-        for line in lines:
-           if line.find("server_args") != -1:
-              tokens = line.split(None)
-              mark = False
-              for t in tokens:
-                 if t == "-s":    
-                    mark = True
-                 elif mark:
-                    return t
-
-    # otherwise, guess based on the distro
+    """
+    Guesses the location of the tftpboot directory,
+    based on the distro on which cobblerd is running
+    """
     (make,version) = os_release()
     if make == "fedora" and version >= 9:
         return "/var/lib/tftpboot"
-    elif make =="redhat" and version >= 6:
+    elif make in ("redhat","centos") and version >= 6:
         return "/var/lib/tftpboot"
-    elif make == "debian" or make == "ubuntu":
-        return "/var/lib/tftpboot"
-    if make == "suse":
+    elif make == "suse":
         return "/srv/tftpboot"
-    return "/tftpboot"
+    # As of Ubuntu 12.04, while they seem to have settled on sticking with 
+    # /var/lib/tftpboot, they haven't scrubbed all of the packages that came 
+    # from Debian that use /srv/tftp by default.
+    elif make == "ubuntu" and os.path.exists("/var/lib/tftpboot"):
+        return "/var/lib/tftpboot"
+    elif make == "ubuntu" and os.path.exists("/srv/tftp"):
+        return "/srv/tftp"
+    elif make == "debian" and int(version.split('.')[0]) < 6:
+        return "/var/lib/tftpboot"
+    elif make == "debian" and int(version.split('.')[0]) >= 6:
+        return "/srv/tftp"
+    else:
+        return "/tftpboot"
 
 def can_do_public_content(api):
     """
@@ -1282,7 +1311,7 @@ def mkdir(path,mode=0755,logger=None):
        if not oe.errno == 17: # already exists (no constant for 17?)
            if logger is not None:
                log_exc(logger)
-           raise CX(_("Error creating") % path)
+           raise CX(_("Error creating %s") % path)
 
 def path_tail(apath, bpath):
     """
@@ -1290,8 +1319,8 @@ def path_tail(apath, bpath):
     """
     position = bpath.find(apath)
     if position != 0:
-        die(self.logger, "- warning: possible symlink traversal?: %s")
-    rposition = position + len(self.mirror)
+        return ""
+    rposition = position + len(apath)
     result = bpath[rposition:]
     if not result.startswith("/"):
         result = "/" + result
@@ -1310,9 +1339,9 @@ def set_arch(self,arch,repo=False):
        arch = "i386"
 
    if repo:
-       valids = [ "i386", "x86_64", "ia64", "ppc", "ppc64", "s390", "s390x", "noarch", "src" ]
+       valids = [ "i386", "x86_64", "ia64", "ppc", "ppc64", "s390", "s390x", "noarch", "src", "arm" ]
    else:
-       valids = [ "i386", "x86_64", "ia64", "ppc", "ppc64", "s390", "s390x" ]
+       valids = [ "i386", "x86_64", "ia64", "ppc", "ppc64", "s390", "s390x", "arm" ]
 
    if arch in valids:
        self.arch = arch
@@ -1327,9 +1356,9 @@ def set_os_version(self,os_version):
    self.os_version = os_version.lower()
    if self.breed is None or self.breed == "":
       raise CX(_("cannot set --os-version without setting --breed first"))
-   if not self.breed in codes.VALID_OS_BREEDS:
+   if not self.breed in get_valid_breeds():
       raise CX(_("fix --breed first before applying this setting"))
-   matched = codes.VALID_OS_VERSIONS[self.breed]
+   matched = SIGNATURE_CACHE["breeds"][self.breed]
    if not os_version in matched:
       nicer = ", ".join(matched)
       raise CX(_("--os-version for breed %s must be one of %s, given was %s") % (self.breed, nicer, os_version))
@@ -1337,7 +1366,7 @@ def set_os_version(self,os_version):
    return True
 
 def set_breed(self,breed):
-   valid_breeds = codes.VALID_OS_BREEDS
+   valid_breeds = get_valid_breeds()
    if breed is not None and breed.lower() in valid_breeds:
        self.breed = breed.lower()
        return True
@@ -1417,7 +1446,7 @@ def set_virt_file_size(self,num):
     try:
         inum = int(num)
         if inum != float(num):
-            return CX(_("invalid virt file size (%s)" % num))
+            raise CX(_("invalid virt file size (%s)" % num))
         if inum >= 0:
             self.virt_file_size = inum
             return True
@@ -1426,6 +1455,20 @@ def set_virt_file_size(self,num):
         raise CX(_("invalid virt file size (%s)" % num))
     return True
 
+def set_virt_disk_driver(self,driver):
+    """
+    For Virt only.
+    Specifies the on-disk format for the virtualized disk
+    """
+    # FIXME: we should probably check the driver type
+    #        here against the libvirt/virtinst list of
+    #        drivers, but this makes things more flexible
+    #        meaning we don't have to manage this list
+    #        and it's up to the user not to enter an
+    #        unsupported disk format
+    self.virt_disk_driver = driver
+    return True
+
 def set_virt_auto_boot(self,num):
     """
     For Virt only.
@@ -1443,9 +1486,27 @@ def set_virt_auto_boot(self,num):
         if (inum == 0) or (inum == 1):
             self.virt_auto_boot = inum
             return True
-        return CX(_("invalid virt_auto_boot value (%s): value must be either '0' (disabled) or '1' (enabled)" % inum))
+        raise CX(_("invalid virt_auto_boot value (%s): value must be either '0' (disabled) or '1' (enabled)" % inum))
     except:
-        return CX(_("invalid virt_auto_boot value (%s): value must be either '0' (disabled) or '1' (enabled)" % inum))
+        raise CX(_("invalid virt_auto_boot value (%s): value must be either '0' (disabled) or '1' (enabled)" % num))
+    return True
+
+def set_virt_pxe_boot(self,num):
+    """
+    For Virt only.
+    Specifies whether the VM should use PXE for booting
+    0 tells Koan not to PXE boot virtuals
+    """
+
+    # num is a non-negative integer (0 means default)
+    try:
+        inum = int(num)
+        if (inum == 0) or (inum == 1):
+            self.virt_pxe_boot = inum
+            return True
+        raise CX(_("invalid virt_pxe_boot value (%s): value must be either '0' (disabled) or '1' (enabled)" % inum))
+    except:
+        raise CX(_("invalid virt_pxe_boot value (%s): value must be either '0' (disabled) or '1' (enabled)" % num))
     return True
 
 def set_virt_ram(self,num):
@@ -1463,13 +1524,13 @@ def set_virt_ram(self,num):
     try:
         inum = int(num)
         if inum != float(num):
-            return CX(_("invalid virt ram size (%s)" % num))
+            raise CX(_("invalid virt ram size (%s)" % num))
         if inum >= 0:
             self.virt_ram = inum
             return True
-        return CX(_("invalid virt ram size (%s)" % num))
+        raise CX(_("invalid virt ram size (%s)" % num))
     except:
-        return CX(_("invalid virt ram size (%s)" % num))
+        raise CX(_("invalid virt ram size (%s)" % num))
     return True
 
 def set_virt_type(self,vtype):
@@ -1481,7 +1542,7 @@ def set_virt_type(self,vtype):
         self.virt_type == "<<inherit>>"
         return True
 
-    if vtype.lower() not in [ "qemu", "xenpv", "xenfv", "vmware", "vmwarew", "auto" ]:
+    if vtype.lower() not in [ "qemu", "kvm", "xenpv", "xenfv", "vmware", "vmwarew", "openvz", "auto" ]:
         raise CX(_("invalid virt type (%s)" % vtype))
     self.virt_type = vtype
     return True
@@ -1662,29 +1723,34 @@ def is_remote_file(file):
     else:
        return False
 
-def subprocess_sp(logger, cmd, shell=True):
+def subprocess_sp(logger, cmd, shell=True, input=None):
     if logger is not None:
         logger.info("running: %s" % cmd)
+
+    stdin = None
+    if input:
+        stdin = sub_process.PIPE
+
     try:
-        sp = sub_process.Popen(cmd, shell=shell, stdout=sub_process.PIPE, stderr=sub_process.PIPE, close_fds=True)
+        sp = sub_process.Popen(cmd, shell=shell, stdin=stdin, stdout=sub_process.PIPE, stderr=sub_process.PIPE, close_fds=True)
     except OSError:
         if logger is not None:
             log_exc(logger)
         die(logger, "OS Error, command not found?  While running: %s" % cmd)
 
-    (out,err) = sp.communicate()
+    (out,err) = sp.communicate(input)
     rc = sp.returncode
     if logger is not None:
         logger.info("received on stdout: %s" % out)
         logger.debug("received on stderr: %s" % err)
     return out, rc
 
-def subprocess_call(logger, cmd, shell=True):
-    data, rc = subprocess_sp(logger, cmd, shell=shell)
+def subprocess_call(logger, cmd, shell=True, input=None):
+    data, rc = subprocess_sp(logger, cmd, shell=shell, input=input)
     return rc
 
-def subprocess_get(logger, cmd, shell=True):
-    data, rc = subprocess_sp(logger, cmd, shell=shell)
+def subprocess_get(logger, cmd, shell=True, input=None):
+    data, rc = subprocess_sp(logger, cmd, shell=shell, input=input)
     return data
 
 def popen2(args, **kwargs):
@@ -1775,9 +1841,12 @@ def from_datastruct_from_fields(obj, seed_data, fields):
         # we don't have to load interface fields here
         if elems[0].startswith("*") or elems[0].find("widget") != -1:
             continue
-        k = elems[0]
-        if seed_data.has_key(k):
-            setattr(obj, k, seed_data[k])
+        src_k = dst_k = elems[0]
+        # deprecated field switcheroo
+        if field_info.DEPRECATED_FIELDS.has_key(src_k):
+            dst_k = field_info.DEPRECATED_FIELDS[src_k]
+        if seed_data.has_key(src_k):
+            setattr(obj, dst_k, seed_data[src_k])
 
     if obj.uid == '':
         obj.uid = obj.config.generate_uid()
@@ -1785,6 +1854,13 @@ def from_datastruct_from_fields(obj, seed_data, fields):
     # special handling for interfaces
     if obj.COLLECTION_TYPE == "system":
         obj.interfaces = copy.deepcopy(seed_data["interfaces"])
+        # deprecated field switcheroo for interfaces
+        for interface in obj.interfaces.keys():
+            for k in obj.interfaces[interface].keys():
+                if field_info.DEPRECATED_FIELDS.has_key(k):
+                    if not obj.interfaces[interface].has_key(field_info.DEPRECATED_FIELDS[k]) or \
+                           obj.interfaces[interface][field_info.DEPRECATED_FIELDS[k]] == "":
+                        obj.interfaces[interface][field_info.DEPRECATED_FIELDS[k]] = obj.interfaces[interface][k]
 
     return obj
 
@@ -1811,6 +1887,10 @@ def to_datastruct_from_fields(obj, fields):
     # they are the only exception in Cobbler.
     if obj.COLLECTION_TYPE == "system":
         ds["interfaces"] = copy.deepcopy(obj.interfaces)
+        #for interface in ds["interfaces"].keys():
+        #    for k in ds["interfaces"][interface].keys():
+        #        if field_info.DEPRECATED_FIELDS.has_key(k):
+        #            ds["interfaces"][interface][field_info.DEPRECATED_FIELDS[k]] = ds["interfaces"][interface][k]
 
     return ds
 
@@ -1858,52 +1938,64 @@ def matches_args(args, list_of):
     return False
 
 def add_options_from_fields(object_type, parser, fields, object_action):
-    for elem in fields:
-        k = elem[0] 
-        if k.find("widget") != -1:
-            continue
-        # scrub interface tags so all fields get added correctly.
-        k = k.replace("*","")
-        default = elem[1]
-        nicename = elem[3]
-        tooltip = elem[5]
-        choices = elem[6]
-        if field_info.ALTERNATE_OPTIONS.has_key(k):
-            niceopt = field_info.ALTERNATE_OPTIONS[k]
-        else:
-            niceopt = "--%s" % k.replace("_","-")
-        desc = nicename
-        if tooltip != "":
-            desc = nicename + " (%s)" % tooltip
+    if object_action in ["add","edit","find","copy","rename"]:
+        for elem in fields:
+            k = elem[0] 
+            if k.find("widget") != -1:
+                continue
+            # scrub interface tags so all fields get added correctly.
+            k = k.replace("*","")
+            default = elem[1]
+            nicename = elem[3]
+            tooltip = elem[5]
+            choices = elem[6]
+            if field_info.ALTERNATE_OPTIONS.has_key(k):
+                niceopt = field_info.ALTERNATE_OPTIONS[k]
+            else:
+                niceopt = "--%s" % k.replace("_","-")
+            desc = nicename
+            if tooltip != "":
+                desc = nicename + " (%s)" % tooltip
 
+            aliasopt = []
+            for deprecated_field in field_info.DEPRECATED_FIELDS.keys():
+                if field_info.DEPRECATED_FIELDS[deprecated_field] == k:
+                    aliasopt.append("--%s" % deprecated_field)
 
-        if isinstance(choices, list) and len(choices) != 0:
-            desc = desc + " (valid options: %s)" % ",".join(choices)    
-            parser.add_option(niceopt, dest=k, help=desc, choices=choices)
-        else:
-            parser.add_option(niceopt, dest=k, help=desc)
+            if isinstance(choices, list) and len(choices) != 0:
+                desc = desc + " (valid options: %s)" % ",".join(choices)    
+                parser.add_option(niceopt, dest=k, help=desc, choices=choices)
+                for alias in aliasopt:
+                    parser.add_option(alias, dest=k, help=desc, choices=choices)
+            else:
+                parser.add_option(niceopt, dest=k, help=desc)
+                for alias in aliasopt:
+                    parser.add_option(alias, dest=k, help=desc)
 
+        if object_type == "system":
+            # system object
+            parser.add_option("--interface", dest="interface", help="the interface to operate on (can only be specified once per command line)")
+            if object_action in ["add","edit"]:
+                parser.add_option("--delete-interface", dest="delete_interface", action="store_true")
+                parser.add_option("--rename-interface", dest="rename_interface")
+
+        if object_action in ["copy","rename"]:
+            parser.add_option("--newname", help="new object name")
+
+        if object_action not in ["find",] and object_type != "setting": 
+            parser.add_option("--clobber", dest="clobber", help="allow add to overwrite existing objects", action="store_true")
+            parser.add_option("--in-place", action="store_true", default=False, dest="in_place", help="edit items in kopts or ksmeta without clearing the other items")
+
+    elif object_action == "remove":
+        parser.add_option("--name", help="%s name to remove" % object_type)
+        parser.add_option("--recursive", action="store_true", dest="recursive", help="also delete child objects")
 
-    # FIXME: not supported in 2.0?
-    if not object_action in ["dumpvars","find","remove","report","list"]: 
-        # FIXME: implement
-        parser.add_option("--clobber", dest="clobber", help="allow add to overwrite existing objects", action="store_true")
-        parser.add_option("--in-place", action="store_true", default=False, dest="in_place", help="edit items in kopts or ksmeta without clearing the other items")
-    if object_action in ["copy","rename"]:
-        parser.add_option("--newname", help="new object name")
     # FIXME: not supported in 2.0 ?
     #if not object_action in ["dumpvars","find","remove","report","list"]: 
     #    parser.add_option("--no-sync",     action="store_true", dest="nosync", help="suppress sync for speed")
     # FIXME: not supported in 2.0 ?
     # if not matches_args(args,["dumpvars","report","list"]):
     #    parser.add_option("--no-triggers", action="store_true", dest="notriggers", help="suppress trigger execution")
-    if object_action in ["remove"]:
-        parser.add_option("--recursive", action="store_true", dest="recursive", help="also delete child objects")
-    if object_type == "system":
-        # system object
-        parser.add_option("--interface", dest="interface", help="which interface to edit")
-        parser.add_option("--delete-interface", dest="delete_interface", action="store_true")
-
 
 def get_remote_methods_from_fields(obj,fields):
     """
@@ -1917,6 +2009,7 @@ def get_remote_methods_from_fields(obj,fields):
     if obj.COLLECTION_TYPE == "system":
        ds["modify_interface"] = getattr(obj,"modify_interface")
        ds["delete_interface"] = getattr(obj,"delete_interface")
+       ds["rename_interface"] = getattr(obj,"rename_interface")
     return ds
 
 def get_power_types():
@@ -1924,9 +2017,11 @@ def get_power_types():
     Return all possible power types
     """
     power_types = []
-    power_template = re.compile(r'power_(.*).template')
-    for x in glob.glob("/etc/cobbler/power/power_*.template"):
+    power_template = re.compile(r'fence_(.*)')
+    fence_files = glob.glob("/usr/sbin/fence_*") + glob.glob("/sbin/fence_*")
+    for x in fence_files:
         power_types.append(power_template.search(x).group(1))
+    power_types.sort()
     return power_types
 
 def get_power(powertype=None):
@@ -1934,11 +2029,65 @@ def get_power(powertype=None):
     Return power command for type
     """
     if powertype:
-        powerpath = "/etc/cobbler/power/power_%s.template" % powertype
-        if os.path.isfile(powerpath):
-            return powerpath
+        # try /sbin, then /usr/sbin
+        powerpath1 = "/sbin/fence_%s" % powertype
+        powerpath2 = "/usr/sbin/fence_%s" % powertype
+        for powerpath in (powerpath1,powerpath2):
+            if os.path.isfile(powerpath) and os.access(powerpath, os.X_OK):
+                return powerpath
     return None
 
+def get_power_template(powertype=None):
+    """
+    Return power template for type
+    """
+    if powertype:
+        powertemplate = "/etc/cobbler/power/fence_%s.template" % powertype
+        if os.path.isfile(powertemplate):
+            f = open(powertemplate)
+            template = f.read()
+            f.close()
+            return template
+    # return a generic template if a specific one wasn't found
+    return "action=$power_mode\nlogin=$power_user\npasswd=$power_pass\nipaddr=$power_address\nport=$power_id"
+
+def load_signatures(filename,cache=True):
+    """
+    Loads the import signatures for distros
+    """
+    global SIGNATURE_CACHE
+    try:
+        f = open(filename,"r")
+        sigjson = f.read()
+        f.close()
+        sigdata = simplejson.loads(sigjson)
+        if cache:
+            SIGNATURE_CACHE = sigdata
+        return True
+    except:
+        return False
+
+def get_valid_breeds():
+    """
+    Return a list of valid breeds found in the import signatures
+    """
+    if SIGNATURE_CACHE.has_key("breeds"):
+        return SIGNATURE_CACHE["breeds"].keys()
+    else:
+        return []
+
+def get_valid_os_versions():
+    """
+    Return a list of valid os-versions found in the import signatures
+    """
+    os_versions = []
+    try:
+        for breed in get_valid_breeds():
+            os_versions += SIGNATURE_CACHE["breeds"][breed].keys()
+    except:
+        pass
+    return uniquify(os_versions)
+
 def get_shared_secret():
     """
     The 'web.ss' file is regenerated each time cobblerd restarts and is
@@ -1959,12 +2108,22 @@ def local_get_cobbler_api_url():
     # Load server and http port
     try:
         fh = open("/etc/cobbler/settings")
-        data = yaml.load(fh.read())
+        data = yaml.safe_load(fh.read())
         fh.close()
     except:
        traceback.print_exc()
        raise CX("/etc/cobbler/settings is not a valid YAML file")
-    return "http://%s:%s/cobbler_api" % (data.get("server","127.0.0.1"),data.get("http_port","80"))
+
+    ip = data.get("server","127.0.0.1")
+    if data.get("client_use_localhost", False):
+        # this overrides the server setting 
+        ip = "127.0.0.1"
+    port = data.get("http_port","80")
+    protocol = "http"
+    if data.get("client_use_https", False):
+        protocol = "https"
+
+    return "%s://%s:%s/cobbler_api" % (protocol,ip,port)
 
 def get_ldap_template(ldaptype=None):
     """
@@ -1980,7 +2139,7 @@ def local_get_cobbler_xmlrpc_url():
     # Load xmlrpc port
     try:
         fh = open("/etc/cobbler/settings")
-        data = yaml.load(fh.read())
+        data = yaml.safe_load(fh.read())
         fh.close()
     except:
        traceback.print_exc()
@@ -2058,24 +2217,73 @@ def dhcpconf_location(api):
         return "/etc/dhcpd.conf"
     elif dist == "suse":
         return "/etc/dhcpd.conf"
+    elif dist == "debian" and int(version[1].split('.')[0]) < 6:
+        return "/etc/dhcp3/dhcpd.conf"
+    elif dist == "ubuntu" and version[1] < 11.10:
+        return "/etc/dhcp3/dhcpd.conf"
     else:
         return "/etc/dhcp/dhcpd.conf"
 
+def namedconf_location(api):
+    (dist, ver) = api.os_version
+    if dist == "debian" or dist == "ubuntu":
+        return "/etc/bind/named.conf"
+    else:
+        return "/etc/named.conf"
+
+def zonefile_base(api):
+    (dist, version) = api.os_version
+    if dist == "debian" or dist == "ubuntu":
+        return "/etc/bind/db."
+    else:
+        return "/var/named/"
+
+def dhcp_service_name(api):
+    (dist, version) = api.os_version
+    if dist == "debian" and int(version.split('.')[0]) < 6:
+        return "dhcp3-server"
+    elif dist == "debian" and int(version.split('.')[0]) >= 6:
+        return "isc-dhcp-server"
+    elif dist == "ubuntu" and version < 11.10:
+        return "dhcp3-server"
+    elif dist == "ubuntu" and version >= 11.10:
+        return "isc-dhcp-server"
+    else:
+        return "dhcpd"
+
+def named_service_name(api):
+    (dist, ver) = api.os_version
+    if dist == "debian" or dist == "ubuntu":
+        return "bind9"
+    else:
+        return "named"
+
 def link_distro(settings, distro):
     # find the tree location
-    base = os.path.join(settings.webdir, "ks_mirror", distro.name)
+    base = find_distro_path(settings, distro)
+    if not base:
+        return
+
     dest_link = os.path.join(settings.webdir, "links", distro.name)
 
     # create the links directory only if we are mirroring because with
     # SELinux Apache can't symlink to NFS (without some doing)
 
-    if not os.path.exists(dest_link):
+    if not os.path.lexists(dest_link):
         try:
             os.symlink(base, dest_link)
         except:
             # this shouldn't happen but I've seen it ... debug ...
             print _("- symlink creation failed: %(base)s, %(dest)s") % { "base" : base, "dest" : dest_link }
 
+def find_distro_path(settings, distro):
+    possible_dirs = glob.glob(settings.webdir+"/ks_mirror/*")
+    for dir in possible_dirs:
+        if os.path.dirname(distro.kernel).find(dir) != -1:
+            return os.path.join(settings.webdir, "ks_mirror", dir)
+    # non-standard directory, assume it's the same as the
+    # directory in which the given distro's kernel is
+    return os.path.dirname(distro.kernel)
 
 if __name__ == "__main__":
     print os_release() # returns 2, not 3
