@@ -5,8 +5,8 @@ a tool for network provisioning of virtualization (xen,kvm/qemu,vmware)
 and network re-provisioning of existing Linux systems.  
 used with 'cobbler'. see manpage for usage.
 
-Copyright 2006-2008 Red Hat, Inc.
-Michael DeHaan <mdehaan@redhat.com>
+Copyright 2006-2008 Red Hat, Inc and Others.
+Michael DeHaan <michael.dehaan AT gmail>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -84,6 +84,7 @@ DISPLAY_PARAMS = [
    "virt_type",
    "virt_path",
    "virt_auto_boot",
+   "virt_pxe_boot",
 ]
 
 
@@ -183,6 +184,10 @@ def main():
                  action="store_true", 
                  dest="virt_auto_boot",
                  help="set VM for autoboot")
+    p.add_option("", "--virt-pxe-boot",
+                 action="store_true",
+                 dest="virt_pxe_boot",
+                 help="PXE boot for installation override")
     p.add_option("", "--add-reinstall-entry",
                  dest="add_reinstall_entry",
                  action="store_true",
@@ -194,7 +199,11 @@ def main():
     p.add_option("", "--kexec",
                  dest="use_kexec",
                  action="store_true",
-                 help="Instead of writing a new bootloader config when using --replace_self, just kexec the new kernel and initrd")
+                 help="Instead of writing a new bootloader config when using --replace-self, just kexec the new kernel and initrd")
+    p.add_option("", "--no-copy-default",
+                 dest="no_copy_default",
+                 action="store_true",
+                 help="Do not copy the kernel args from the default kernel entry when using --replace-self")
     p.add_option("", "--embed",
                  dest="embed_kickstart",
                  action="store_true",
@@ -205,6 +214,20 @@ def main():
     p.add_option("", "--qemu-net-type",
                  dest="qemu_net_type",
                  help="when used with --virt_type=qemu, select type of network device to use: e1000, ne2k_pci, pcnet, rtl8139, virtio")
+    p.add_option("", "--qemu-machine-type",
+                 dest="qemu_machine_type",
+                 help="when used with --virt_type=qemu, select type of machine type to emulate: pc, pc-1.0, pc-0.15")
+    p.add_option("", "--wait",
+                 dest="wait", type='int', default=0,	# default to 0 for koan backwards compatibility
+                 help="pass the --wait=<INT> argument to virt-install")
+    p.add_option("", "--noreboot",
+                 dest="noreboot", default=False,	# default to False for koan backwards compatibility
+                 action="store_true",
+                 help="pass the --noreboot argument to virt-install")
+    p.add_option("", "--import",
+                 dest="osimport", default=False,	# default to False for koan backwards compatibility
+                 action="store_true",
+                 help="pass the --import argument to virt-install")
 
     (options, args) = p.parse_args()
 
@@ -232,11 +255,17 @@ def main():
         k.kopts_override      = options.kopts_override
         k.static_interface    = options.static_interface
         k.use_kexec           = options.use_kexec
+        k.no_copy_default     = options.no_copy_default
         k.should_poll         = options.should_poll
         k.embed_kickstart     = options.embed_kickstart
         k.virt_auto_boot      = options.virt_auto_boot
+        k.virt_pxe_boot       = options.virt_pxe_boot
         k.qemu_disk_type      = options.qemu_disk_type
         k.qemu_net_type       = options.qemu_net_type
+        k.qemu_machine_type   = options.qemu_machine_type
+        k.virtinstall_wait    = options.wait
+        k.virtinstall_noreboot= options.noreboot
+        k.virtinstall_osimport= options.osimport
 
         if options.virt_name is not None:
             k.virt_name          = options.virt_name
@@ -295,13 +324,18 @@ class Koan:
         self.force_path        = None
         self.qemu_disk_type    = None
         self.qemu_net_type     = None
+        self.qemu_machine_type = None
         self.virt_auto_boot    = None
+        self.virt_pxe_boot     = None
+        self.virtinstall_wait  = None
+        self.virtinstall_noreboot = None
+        self.virtinstall_osimport = None
 
         # This option adds the --copy-default argument to /sbin/grubby
         # which uses the default boot entry in the grub.conf
         # as template for the new entry being added to that file.
         # look at /sbin/grubby --help for more info
-        self.grubby_copy_default  =  1
+        self.no_copy_default   = None
 
     #---------------------------------------------------
 
@@ -357,23 +391,28 @@ class Koan:
         # if --virt-type was specified and invalid, then fail
         if self.virt_type is not None:
             self.virt_type = self.virt_type.lower()
-            if self.virt_type not in [ "qemu", "xenpv", "xenfv", "xen", "vmware", "vmwarew", "auto" ]:
+            if self.virt_type not in [ "qemu", "xenpv", "xenfv", "xen", "vmware", "vmwarew", "auto", "kvm" ]:
                if self.virt_type == "xen":
                    self.virt_type = "xenpv"
-               raise InfoException, "--virt-type should be qemu, xenpv, xenfv, vmware, vmwarew, or auto"
+               raise InfoException, "--virt-type should be qemu, xenpv, xenfv, vmware, vmwarew, kvm, or auto"
 
         # if --qemu-disk-type was called without --virt-type=qemu, then fail
         if (self.qemu_disk_type is not None):
             self.qemu_disk_type = self.qemu_disk_type.lower()
-            if self.virt_type not in [ "qemu", "auto" ]:
+            if self.virt_type not in [ "qemu", "auto", "kvm" ]:
                raise InfoException, "--qemu-disk-type must use with --virt-type=qemu"
 
         # if --qemu-net-type was called without --virt-type=qemu, then fail
         if (self.qemu_net_type is not None):
             self.qemu_net_type = self.qemu_net_type.lower()
-            if self.virt_type not in [ "qemu", "auto" ]:
+            if self.virt_type not in [ "qemu", "auto", "kvm" ]:
                raise InfoException, "--qemu-net-type must use with --virt-type=qemu"
 
+        # if --qemu-machine-type was called without --virt-type=qemu, then fail
+        if (self.qemu_machine_type is not None):
+            self.qemu_machine_type = self.qemu_machine_type.lower()
+            if self.virt_type not in [ "qemu", "auto", "kvm" ]:
+               raise InfoException, "--qemu-machine-type must use with --virt-type=qemu"
 
 
         # if --static-interface and --profile was called together, then fail
@@ -574,8 +613,8 @@ class Koan:
                 cmd = sub_process.Popen("uname -r", stdout=sub_process.PIPE, shell=True)
                 uname_str = cmd.communicate()[0]
                 # correct kernel on dom0?
-                if uname_str.find("xen") == -1:
-                   raise InfoException("kernel-xen needs to be in use")
+                if uname_str < "2.6.37" and uname_str.find("xen") == -1:
+                   raise InfoException("kernel >= 2.6.37 or kernel-xen needs to be in use")
                 # xend installed?
                 if not os.path.exists("/usr/sbin/xend"):
                    raise InfoException("xen package needs to be installed")
@@ -585,7 +624,7 @@ class Koan:
                    raise InfoException("xend needs to be started")
 
             # for qemu
-            if self.virt_type == "qemu":
+            if self.virt_type in [ "qemu", "kvm" ]:
                 # qemu package installed?
                 if not os.path.exists("/usr/bin/qemu-img"):
                     raise InfoException("qemu package needs to be installed")
@@ -640,6 +679,7 @@ class Koan:
         if profile_data["breed"] == "suse":
             kopts = profile_data["kernel_options"]
             options = kopts.split(" ")
+            profile_data["install_tree"] = ""
             for opt in options:
                 if opt.startswith("install="):
                     profile_data["install_tree"] = opt.replace("install=","")
@@ -693,8 +733,29 @@ class Koan:
         """
 
         try:
-            tree = profile_data["ks_meta"].split("@@")[-1].strip()
-            profile_data["install_tree"] = "http://" + profile_data["http_server"] + tree
+            tree = profile_data["ks_meta"].split()
+            # Ensure we only take the tree in case ks_meta args are passed
+            # First check for tree= in ks_meta arguments
+            meta_re=re.compile('tree=')
+            tree_found=''
+            for entry in tree:
+                if meta_re.match(entry):
+                    tree_found=entry.split("=")[-1]
+                    break
+ 
+            if tree_found=='':
+                # assume tree information as first argument
+                tree = tree.split()[0]
+            else:
+                tree=tree_found
+            tree_re = re.compile ('(http|ftp|nfs):')
+            # Next check for installation tree on remote server
+            if tree_re.match(tree):
+                tree = tree.replace("@@http_server@@", profile_data["http_server"])
+                profile_data["install_tree"] = tree
+            else:
+            # Now take the first parameter as the local path
+                profile_data["install_tree"] = "http://" + profile_data["http_server"] + tree
 
             if self.safe_load(profile_data,"install_tree"):
                 print "install_tree:", profile_data["install_tree"]
@@ -748,7 +809,7 @@ class Koan:
         system for folks who are not needing a more complex CMS.
         
         Read more at:
-        https://fedorahosted.org/cobbler/wiki/BuiltinConfigManagement
+        https://github.com/cobbler/cobbler/wiki/Built-in-configuration-management
         """
 
         # FIXME: make this a utils.py function
@@ -877,7 +938,7 @@ class Koan:
 
             (make, version) = utils.os_release()
 
-            if (make == "centos" and version < 6) or (make == "redhat" and version < 6) or (make == "fedora" and version < 10):
+            if (make == "centos" and version < 7) or (make == "redhat" and version < 7) or (make == "fedora" and version < 10):
 
                 # embed the initrd in the kickstart file because of libdhcp and/or pump
                 # needing the help due to some DHCP timeout potential in some certain
@@ -897,14 +958,15 @@ class Koan:
             #   asm-powerpc/setup.h:#define COMMAND_LINE_SIZE   512
             #   asm-s390/setup.h:#define COMMAND_LINE_SIZE  896
             #   asm-x86_64/setup.h:#define COMMAND_LINE_SIZE    256
+            #   arch/x86/include/asm/setup.h:#define COMMAND_LINE_SIZE 2048
             if arch.startswith("ppc") or arch.startswith("ia64"):
                 if len(k_args) > 511:
                     raise InfoException, "Kernel options are too long, 512 chars exceeded: %s" % k_args
             elif arch.startswith("s390"):
                 if len(k_args) > 895:
                     raise InfoException, "Kernel options are too long, 896 chars exceeded: %s" % k_args
-            elif len(k_args) > 255:
-                raise InfoException, "Kernel options are too long, 255 chars exceeded: %s" % k_args
+            elif len(k_args) > 2048:
+                raise InfoException, "Kernel options are too long, 2048 chars exceeded: %s" % k_args
 
             utils.subprocess_call([
                 'kexec',
@@ -950,15 +1012,23 @@ class Koan:
     
 
         def after_download(self, profile_data):
-            if not os.path.exists("/sbin/grubby"):
-                raise InfoException, "grubby is not installed"
+            use_grubby = False
+            use_grub2 = False
+            (make, version) = utils.os_release()
+            if make in ['ubuntu', 'debian']:
+                if not os.path.exists("/usr/sbin/update-grub"):
+                    raise InfoException, "grub2 is not installed"
+                use_grub2 = True
+            else:
+                if not os.path.exists("/sbin/grubby"):
+                    raise InfoException, "grubby is not installed"
+                use_grubby = True
+
             k_args = self.calc_kernel_args(profile_data,replace_self=1)
 
             kickstart = self.safe_load(profile_data,'kickstart')
 
-            (make, version) = utils.os_release()
-
-            if (make == "centos" and version < 6) or (make == "redhat" and version < 6) or (make == "fedora" and version < 10):
+            if (make == "centos" and version < 7) or (make == "redhat" and version < 7) or (make == "fedora" and version < 10):
 
                 # embed the initrd in the kickstart file because of libdhcp and/or pump
                 # needing the help due to some DHCP timeout potential in some certain
@@ -984,6 +1054,7 @@ class Koan:
             #   asm-powerpc/setup.h:#define COMMAND_LINE_SIZE   512
             #   asm-s390/setup.h:#define COMMAND_LINE_SIZE  896
             #   asm-x86_64/setup.h:#define COMMAND_LINE_SIZE    256
+            #   arch/x86/include/asm/setup.h:#define COMMAND_LINE_SIZE 2048
             if not ANCIENT_PYTHON:
                 if arch.startswith("ppc") or arch.startswith("ia64"):
                     if len(k_args) > 511:
@@ -991,59 +1062,103 @@ class Koan:
                 elif arch.startswith("s390"):
                     if len(k_args) > 895:
                         raise InfoException, "Kernel options are too long, 896 chars exceeded: %s" % k_args
-                elif len(k_args) > 255:
-                    raise InfoException, "Kernel options are too long, 255 chars exceeded: %s" % k_args
+                elif len(k_args) > 2048:
+                    raise InfoException, "Kernel options are too long, 2048 chars exceeded: %s" % k_args
 
-            cmd = [ "/sbin/grubby",
-                    "--add-kernel", self.safe_load(profile_data,'kernel_local'),
-                    "--initrd", self.safe_load(profile_data,'initrd_local'),
-                    "--args", "\"%s\"" % k_args
-            ]
+            if use_grubby:
+                cmd = [ "/sbin/grubby",
+                        "--add-kernel", self.safe_load(profile_data,'kernel_local'),
+                        "--initrd", self.safe_load(profile_data,'initrd_local'),
+                        "--args", "\"%s\"" % k_args
+                ]
 
-            if self.grubby_copy_default:
-                cmd.append("--copy-default")
+                if not self.no_copy_default:
+                    cmd.append("--copy-default")
 
-            boot_probe_ret_code, probe_output = self.get_boot_loader_info()
-            if boot_probe_ret_code == 0 and string.find(probe_output, "lilo") >= 0:
-                cmd.append("--lilo")
+                boot_probe_ret_code, probe_output = self.get_boot_loader_info()
+                if boot_probe_ret_code == 0 and string.find(probe_output, "lilo") >= 0:
+                    cmd.append("--lilo")
 
-            if self.add_reinstall_entry:
-               cmd.append("--title=Reinstall")
-            else:
-               cmd.append("--make-default")
-               cmd.append("--title=kick%s" % int(time.time()))
+                if self.add_reinstall_entry:
+                    cmd.append("--title=Reinstall")
+                else:
+                    cmd.append("--make-default")
+                    cmd.append("--title=kick%s" % int(time.time()))
                
-            if self.live_cd:
-               cmd.append("--bad-image-okay")
-               cmd.append("--boot-filesystem=/")
-               cmd.append("--config-file=/tmp/boot/boot/grub/grub.conf")
+                if self.live_cd:
+                    cmd.append("--bad-image-okay")
+                    cmd.append("--boot-filesystem=/")
+                    cmd.append("--config-file=/tmp/boot/boot/grub/grub.conf")
 
-            # Are we running on ppc?
-            if not ANCIENT_PYTHON:
-               if arch.startswith("ppc"):
-                   cmd.append("--yaboot")
-               elif arch.startswith("s390"):
-                   cmd.append("--zipl")
+                # Are we running on ppc?
+                if not ANCIENT_PYTHON:
+                    if arch.startswith("ppc"):
+                        cmd.append("--yaboot")
+                    elif arch.startswith("s390"):
+                        cmd.append("--zipl")
 
-            utils.subprocess_call(cmd)
-
-            # Any post-grubby processing required (e.g. ybin, zipl, lilo)?
-            if not ANCIENT_PYTHON and arch.startswith("ppc"):
-                # FIXME - CHRP hardware uses a 'PPC PReP Boot' partition and doesn't require running ybin
-                print "- applying ybin changes"
-                cmd = [ "/sbin/ybin" ]
                 utils.subprocess_call(cmd)
-            elif not ANCIENT_PYTHON and arch.startswith("s390"):
-                print "- applying zipl changes"
-                cmd = [ "/sbin/zipl" ]
-                utils.subprocess_call(cmd)
-            else:
-                # if grubby --bootloader-probe returns lilo,
-                #    apply lilo changes
-                if boot_probe_ret_code == 0 and string.find(probe_output, "lilo") != -1:
-                    print "- applying lilo changes"
-                    cmd = [ "/sbin/lilo" ]
+
+                # Any post-grubby processing required (e.g. ybin, zipl, lilo)?
+                if not ANCIENT_PYTHON and arch.startswith("ppc"):
+                    # FIXME - CHRP hardware uses a 'PPC PReP Boot' partition and doesn't require running ybin
+                    print "- applying ybin changes"
+                    cmd = [ "/sbin/ybin" ]
                     utils.subprocess_call(cmd)
+                elif not ANCIENT_PYTHON and arch.startswith("s390"):
+                    print "- applying zipl changes"
+                    cmd = [ "/sbin/zipl" ]
+                    utils.subprocess_call(cmd)
+                else:
+                    # if grubby --bootloader-probe returns lilo,
+                    #    apply lilo changes
+                    if boot_probe_ret_code == 0 and string.find(probe_output, "lilo") != -1:
+                        print "- applying lilo changes"
+                        cmd = [ "/sbin/lilo" ]
+                        utils.subprocess_call(cmd)
+
+            elif use_grub2:
+                # Use grub2 for --replace-self
+                kernel_local = self.safe_load(profile_data,'kernel_local')
+                initrd_local = self.safe_load(profile_data,'initrd_local')
+
+                # Set name for grub2 menuentry
+                if self.add_reinstall_entry:
+                    name = "Reinstall: %s" % profile_data['name']
+                else:
+                    name = "%s" % profile_data['name']
+
+                # Set paths for Ubuntu/Debian
+                # TODO: Add support for other distros when they ship grub2
+                if make in ['ubuntu', 'debian']:
+                    grub_file = "/etc/grub.d/42_koan"
+                    grub_default_file = "/etc/default/grub"
+                    cmd = ["update-grub"]
+                    default_cmd = ['sed', '-i', 's/^GRUB_DEFAULT\=.*$/GRUB_DEFAULT="%s"/g' % name, grub_default_file]
+
+                # Create grub2 menuentry
+                grub_entry = """
+                cat <<EOF
+                menuentry "%s" {
+                    linux %s %s
+                    initrd %s
+                }
+                EOF
+                """ % (name, kernel_local, k_args, initrd_local)
+
+                # Save grub2 menuentry
+                fd = open(grub_file,"w")
+                fd.write(grub_entry)
+                fd.close()
+                os.chmod(grub_file, 0755)
+
+                # Set default grub entry for reboot
+                if not self.add_reinstall_entry:
+                    print "- setting grub2 default entry"
+                    sub_process.call(default_cmd)
+
+                # Run update-grub
+                utils.subprocess_call(cmd)
 
             if not self.add_reinstall_entry:
                 print "- reboot to apply changes"
@@ -1062,7 +1177,7 @@ class Koan:
         return r"""
         cd /var/spool/koan
         mkdir initrd
-        gzip -dc %s > initrd.tmp
+        gzip -dc %s > initrd.tmp || xz -dc %s > initrd.tmp
         if mount -o loop -t ext2 initrd.tmp initrd >&/dev/null ; then
             cp ks.cfg initrd/
             ln initrd/ks.cfg initrd/tmp/ks.cfg
@@ -1167,8 +1282,8 @@ class Koan:
         initrd = self.safe_load(profile_data,'initrd')
         kernel_short = os.path.basename(kernel)
         initrd_short = os.path.basename(initrd)
-        kernel_save = "%s/%s" % (download_root, kernel_short)
-        initrd_save = "%s/%s" % (download_root, initrd_short)
+        kernel_save = "%s/%s_koan" % (download_root, kernel_short)
+        initrd_save = "%s/%s_koan" % (download_root, initrd_short)
 
         if self.server:
             if kernel[0] == "/":
@@ -1202,7 +1317,7 @@ class Koan:
             if breed is not None and breed == "suse":
                 kextra = "autoyast=" + kickstart
             elif breed is not None and breed == "debian" or breed =="ubuntu":
-                kextra = "auto url=" + kickstart
+                kextra = "auto-install/enable=true priority=critical url=" + kickstart
             else:
                 kextra = "ks=" + kickstart 
 
@@ -1215,7 +1330,7 @@ class Koan:
 
         hashv = utils.input_string_or_hash(kextra)
 
-        if self.static_interface is not None and (breed is None or breed == "redhat"):
+        if self.static_interface is not None and (breed == "redhat" or breed == "suse"):
             interface_name = self.static_interface
             interfaces = self.safe_load(pd, "interfaces")
             if interface_name.startswith("eth"):
@@ -1227,14 +1342,26 @@ class Koan:
             ip = self.safe_load(interface_data, "ip_address")
             netmask = self.safe_load(interface_data, "netmask")
             gateway = self.safe_load(pd, "gateway")
+            dns = self.safe_load(pd, "name_servers")
 
-            hashv["ksdevice"] = self.static_interface
+            if breed == "suse":
+                hashv["netdevice"] = self.static_interface
+            else:
+                hashv["ksdevice"] = self.static_interface
             if ip is not None:
-                hashv["ip"] = ip
+                if breed == "suse":
+                    hashv["hostip"] = ip
+                else:
+                    hashv["ip"] = ip
             if netmask is not None:
                 hashv["netmask"] = netmask
             if gateway is not None:
                 hashv["gateway"] = gateway
+            if dns is not None:
+                if breed == "suse":
+                    hashv["nameserver"] = dns[0]
+                else:
+                    hashv["dns"] = ",".join(dns)
 
         if replace_self and self.embed_kickstart:
            hashv["ks"] = "file:ks.cfg"
@@ -1270,28 +1397,37 @@ class Koan:
         path_list           = self.calc_virt_path(pd, virtname)
         size_list           = self.calc_virt_filesize(pd)
         driver_list         = self.calc_virt_drivers(pd)
-        disks               = self.merge_disk_data(path_list,size_list,driver_list)
+        if self.virt_type == 'openvz':
+            disks = None
+        else:
+            disks           = self.merge_disk_data(path_list,size_list,driver_list)
         virt_auto_boot      = self.calc_virt_autoboot(pd, self.virt_auto_boot)
+        virt_pxe_boot       = self.calc_virt_pxeboot(pd, self.virt_pxe_boot)
 
         results = create_func(
-                name             =  virtname,
-                ram              =  ram,
-                disks            =  disks,
-                uuid             =  uuid,
-                extra            =  kextra,
-                vcpus            =  vcpus,
-                profile_data     =  profile_data,
-                arch             =  arch,
-                no_gfx           =  self.no_gfx,
-                fullvirt         =  fullvirt,
-                bridge           =  self.virt_bridge,
-                virt_type        =  self.virt_type,
-                virt_auto_boot   =  virt_auto_boot,
-                qemu_driver_type =  self.qemu_disk_type,
-                qemu_net_type    =  self.qemu_net_type
+                name              =  virtname,
+                ram               =  ram,
+                disks             =  disks,
+                uuid              =  uuid,
+                extra             =  kextra,
+                vcpus             =  vcpus,
+                profile_data      =  profile_data,
+                arch              =  arch,
+                no_gfx            =  self.no_gfx,
+                fullvirt          =  fullvirt,
+                bridge            =  self.virt_bridge,
+                virt_type         =  self.virt_type,
+                virt_auto_boot    =  virt_auto_boot,
+                virt_pxe_boot     =  virt_pxe_boot,
+                qemu_driver_type  =  self.qemu_disk_type,
+                qemu_net_type     =  self.qemu_net_type,
+                qemu_machine_type =  self.qemu_machine_type,
+                wait              =  self.virtinstall_wait,
+                noreboot          =  self.virtinstall_noreboot,
+                osimport          =  self.virtinstall_osimport,
         )
 
-        print results
+        #print results
 
         if can_poll is not None and self.should_poll:
             import libvirt
@@ -1323,9 +1459,12 @@ class Koan:
 
         if virt_auto_boot:
             if self.virt_type in [ "xenpv", "xenfv" ]:
-                utils.create_xendomains_symlink(virtname)
-            elif self.virt_type == "qemu":
-               utils.libvirt_enable_autostart(virtname)
+                if not utils.create_xendomains_symlink(virtname):
+                    print "- warning: failed to setup autoboot for %s, it will have to be configured manually" % virtname
+            elif self.virt_type in [ "qemu", "kvm" ]:
+                utils.libvirt_enable_autostart(virtname)
+            elif self.virt_type in [ "openvz" ]:
+                pass
             else:
                 print "- warning: don't know how to autoboot this virt type yet"
             # else...
@@ -1359,7 +1498,7 @@ class Koan:
             if self.virt_type == "xenfv":
                fullvirt = True 
             can_poll = "xen"
-        elif self.virt_type == "qemu":
+        elif self.virt_type in [ "qemu", "kvm" ] :
             fullvirt = True
             uuid    = None
             import qcreate
@@ -1373,6 +1512,10 @@ class Koan:
             import vmwwcreate
             uuid = None
             creator = vmwwcreate.start_install
+        elif self.virt_type == "openvz":
+            import openvzcreate
+            uuid = None
+            creator = openvzcreate.start_install
         else:
             raise InfoException, "Unspecified virt type: %s" % self.virt_type
         return (uuid, creator, fullvirt, can_poll)
@@ -1388,7 +1531,10 @@ class Koan:
                 size = sizes[-1]
             else:
                 size = sizes[counter]
-            driver = drivers[counter]
+            if counter >= len(drivers): 
+                driver = drivers[-1]
+            else:
+                driver = drivers[counter]
             disks.append([path,size,driver])
             counter = counter + 1
         if len(disks) == 0:
@@ -1432,6 +1578,20 @@ class Koan:
 
     #--------------------------------------------------
 
+    def calc_virt_pxeboot(self,data,override_pxeboot=False):
+        if override_pxeboot:
+           return True
+
+        pxeboot = self.safe_load(data,'virt_pxe_boot',0)
+        pxeboot = str(pxeboot).lower()
+
+        if pxeboot in [ "1", "true", "y", "yes" ]:
+           return True
+
+        return False
+
+    #--------------------------------------------------
+
     def calc_virt_filesize(self,data,default_filesize=0):
 
         # MAJOR FIXME: are there overrides?  
@@ -1474,7 +1634,7 @@ class Koan:
             #        the virtinst VirtualDisk class, but
             #        not all versions of virtinst have a 
             #        nice list to use
-            if t in ('raw','qcow','aio'):
+            if t in ('raw','qcow','qcow2','aio', 'vmdk'):
                accum.append(t)
             else:
                print "invalid disk driver specified, defaulting to 'raw'"
@@ -1565,7 +1725,7 @@ class Koan:
            # not set in cobbler either? then assume reasonable defaults
            if self.virt_type in [ "xenpv", "xenfv" ]:
                prefix = "/var/lib/xen/images/"
-           elif self.virt_type == "qemu":
+           elif self.virt_type in [ "qemu", "kvm" ]:
                prefix = "/var/lib/libvirt/images/"
            elif self.virt_type == "vmwarew":
                prefix = "/var/lib/vmware/%s/" % name
@@ -1597,10 +1757,10 @@ class Koan:
 
         # Parse the command line to determine if this is a 
         # path, a partition, or a volume group parameter
-        #   Ex:   /foo
-        #   Ex:   partition:/dev/foo
-        #   Ex:   volume-group:/dev/foo/
-            
+        #   file Ex:         /foo
+        #   partition Ex:    /dev/foo
+        #   volume-group Ex: vg-name(:lv-name)
+        #
         # chosing the disk image name (if applicable) is somewhat
         # complicated ...
 
@@ -1616,7 +1776,7 @@ class Koan:
                 if self.force_path:
                     return location
                 else:
-		    raise InfoException, "The location %s is an existing file. Consider '--force-path' to overwrite it." % location
+                    raise InfoException, "The location %s is an existing file. Consider '--force-path' to overwrite it." % location
         elif location.startswith("/dev/"):
             # partition
             if os.path.exists(location):
@@ -1625,21 +1785,27 @@ class Koan:
                 raise InfoException, "virt path is not a valid block device"
         else:
             # it's a volume group, verify that it exists
+            if location.find(':') == -1:
+                vgname = location
+                lvname = "%s-disk%s" % (name,offset)
+            else:
+                vgname, lvname = location.split(':')[:2]
+
             args = "vgs -o vg_name"
             print "%s" % args
             vgnames = sub_process.Popen(args, shell=True, stdout=sub_process.PIPE).communicate()[0]
             print vgnames
 
-            if vgnames.find(location) == -1:
-                raise InfoException, "The volume group [%s] does not exist." % location
+            if vgnames.find(vgname) == -1:
+                raise InfoException, "The volume group [%s] does not exist." % vgname
             
             # check free space
-            args = "LANG=C vgs --noheadings -o vg_free --units g %s" % location
+            args = "LANG=C vgs --noheadings -o vg_free --units g %s" % vgname
             print args
             cmd = sub_process.Popen(args, stdout=sub_process.PIPE, shell=True)
             freespace_str = cmd.communicate()[0]
             freespace_str = freespace_str.split("\n")[0].strip()
-            freespace_str = freespace_str.lower().replace("g","") # remove gigabytes
+            freespace_str = freespace_str.lower().replace("g","").replace(',', '.') # remove gigabytes
             print "(%s)" % freespace_str
             freespace = int(float(freespace_str))
 
@@ -1653,23 +1819,27 @@ class Koan:
             if freespace >= int(virt_size):
             
                 # look for LVM partition named foo, create if doesn't exist
-                args = "lvs -o lv_name %s" % location
+                args = "lvs --noheadings -o lv_name %s" % vgname
                 print "%s" % args
                 lvs_str=sub_process.Popen(args, stdout=sub_process.PIPE, shell=True).communicate()[0]
                 print lvs_str
          
-                name = "%s-disk%s" % (name,offset)
- 
                 # have to create it?
-                if lvs_str.find(name) == -1:
-                    args = "lvcreate -L %sG -n %s %s" % (virt_size, name, location)
+                found_lvs = False
+                for lvs in lvs_str.split("\n"):
+                    if lvs.strip() == lvname:
+                        found_lvs = True
+                        break
+
+                if not found_lvs:
+                    args = "lvcreate -L %sG -n %s %s" % (virt_size, lvname, vgname)
                     print "%s" % args
                     lv_create = sub_process.call(args, shell=True)
                     if lv_create != 0:
                         raise InfoException, "LVM creation failed"
 
                 # partition location
-                partition_location = "/dev/mapper/%s-%s" % (location,name.replace('-','--'))
+                partition_location = "/dev/mapper/%s-%s" % (vgname.replace('-','--'),lvname.replace('-','--'))
 
                 # check whether we have SELinux enabled system
                 args = "/usr/sbin/selinuxenabled"
