@@ -4,6 +4,26 @@ from django.template import RequestContext
 from django.http import HttpResponse
 from django.http import HttpResponseRedirect
 from django.shortcuts import render_to_response
+from django.views.decorators.http import require_POST
+
+try:
+    from django.views.decorators.csrf import csrf_protect
+except:
+    # Old Django, fudge the @csrf_protect decorator to be a pass-through 
+    # that does nothing. Django decorator shell based on this page: 
+    # http://passingcuriosity.com/2009/writing-view-decorators-for-django/
+    def csrf_protect(f):
+        def _dec(view_func):
+            def _view(request,*args,**kwargs):
+                return view_func(request,*args,**kwargs)
+            _view.__name__ = view_func.__name__
+            _view.__dict__ = view_func.__dict__
+            _view.__doc__  = view_func.__doc__
+            return _view
+        if f is None:
+            return _dec
+        else:
+            return _dec(f)
 
 import xmlrpclib
 import time
@@ -21,6 +41,7 @@ import cobbler.item_image     as item_image
 import cobbler.item_mgmtclass as item_mgmtclass
 import cobbler.item_package   as item_package
 import cobbler.item_file      as item_file
+import cobbler.settings       as item_settings
 import cobbler.field_info     as field_info
 import cobbler.utils          as utils
 
@@ -34,11 +55,11 @@ def index(request):
    """
    This is the main greeting page for cobbler web.
    """
-   if not test_user_authenticated(request): return login(request,next="/cobbler_web")
+   if not test_user_authenticated(request): return login(request,next="/cobbler_web", expired=True)
 
    t = get_template('index.tmpl')
-   html = t.render(Context({
-        'version' : remote.version(request.session['token']),
+   html = t.render(RequestContext(request,{
+        'version' : remote.extended_version(request.session['token'])['version'],
         'username': username,
    }))
    return HttpResponse(html)
@@ -49,10 +70,10 @@ def task_created(request):
    """
    Let's the user know what to expect for event updates.
    """
-   if not test_user_authenticated(request): return login(request, next="/cobbler_web/task_created")
+   if not test_user_authenticated(request): return login(request, next="/cobbler_web/task_created", expired=True)
    t = get_template("task_created.tmpl")
-   html = t.render(Context({
-       'version'  : remote.version(request.session['token']),
+   html = t.render(RequestContext(request,{
+       'version'  : remote.extended_version(request.session['token'])['version'],
        'username' : username
    }))
    return HttpResponse(html)
@@ -63,14 +84,14 @@ def error_page(request,message):
    """
    This page is used to explain error messages to the user.
    """
-   if not test_user_authenticated(request): return login(request)
+   if not test_user_authenticated(request): return login(request,expired=True)
    # FIXME: test and make sure we use this rather than throwing lots of tracebacks for
    # field errors
    t = get_template('error_page.tmpl')
    message = message.replace("<Fault 1: \"<class 'cobbler.cexceptions.CX'>:'","Remote exception: ")
    message = message.replace("'\">","")
-   html = t.render(Context({
-       'version' : remote.version(request.session['token']),
+   html = t.render(RequestContext(request,{
+       'version' : remote.extended_version(request.session['token'])['version'],
        'message' : message,
        'username': username
    }))
@@ -103,6 +124,8 @@ def get_fields(what, is_subobject, seed_item=None):
         field_data = item_package.FIELDS
     if what == "file":
         field_data = item_file.FIELDS
+    if what == "setting":
+        field_data = item_settings.FIELDS
 
     settings = remote.get_settings()
 
@@ -129,12 +152,13 @@ def get_fields(what, is_subobject, seed_item=None):
             "html_element"            : "generic",
         }
 
-
         if not elem["editable"]:
             continue
 
         if seed_item is not None:
-            if row[0].startswith("*"):
+            if what == "setting":
+                elem["value"] = seed_item[row[0]]
+            elif row[0].startswith("*"):
                 # system interfaces are loaded by javascript, not this
                 elem["value"]             = ""
                 elem["name"]              = row[0].replace("*","")
@@ -171,9 +195,14 @@ def get_fields(what, is_subobject, seed_item=None):
             else:
                 tokens = []
                 for (x,y) in elem["value"].items():
-                   if y is not None:
+                   if isinstance(y,str) and y.strip() != "~":
+                      y = y.replace(" ","\\ ")
                       tokens.append("%s=%s" % (x,y))
-                   else:
+                   elif isinstance(y,list):
+                      for l in y:
+                         l = l.replace(" ","\\ ")
+                         tokens.append("%s=%s" % (x,l))
+                   elif y != None:
                       tokens.append("%s" % x)
                 elem["value"] = " ".join(tokens)
 
@@ -275,7 +304,7 @@ def genlist(request, what, page=None):
     Lists all object types, complete with links to actions
     on those objects.
     """
-    if not test_user_authenticated(request): return login(request, next="/cobbler_web/%s/list" % what)
+    if not test_user_authenticated(request): return login(request, next="/cobbler_web/%s/list" % what, expired=True)
 
     # get details from the session
     if page == None:
@@ -340,14 +369,15 @@ def genlist(request, what, page=None):
         'items'          : __format_items(pageditems["items"],columns),
         'pageinfo'       : pageditems["pageinfo"],
         'filters'        : filters,
-        'version'        : remote.version(request.session['token']),
+        'version'        : remote.extended_version(request.session['token'])['version'],
         'username'       : username,
         'limit'          : limit,
         'batchactions'   : batchactions,
     }))
     return HttpResponse(html)
 
-
+@require_POST
+@csrf_protect
 def modify_list(request, what, pref, value=None):
     """
     This function is used in the generic list view
@@ -357,7 +387,7 @@ def modify_list(request, what, pref, value=None):
     This function modifies the session object to
     store these preferences persistently.
     """
-    if not test_user_authenticated(request): return login(request, next="/cobbler_web/%s/modifylist/%s/%s" % (what,pref,str(value)))
+    if not test_user_authenticated(request): return login(request, next="/cobbler_web/%s/modifylist/%s/%s" % (what,pref,str(value)), expired=True)
 
 
     # what preference are we tweaking?
@@ -414,12 +444,14 @@ def modify_list(request, what, pref, value=None):
 
 # ======================================================================
 
+@require_POST
+@csrf_protect
 def generic_rename(request, what, obj_name=None, obj_newname=None):
 
    """
    Renames an object.
    """
-   if not test_user_authenticated(request): return login(request, next="/cobbler_web/%s/rename/%s/%s" % (what,obj_name,obj_newname))
+   if not test_user_authenticated(request): return login(request, next="/cobbler_web/%s/rename/%s/%s" % (what,obj_name,obj_newname), expired=True)
 
    if obj_name == None:
       return error_page(request,"You must specify a %s to rename" % what)
@@ -434,11 +466,13 @@ def generic_rename(request, what, obj_name=None, obj_newname=None):
 
 # ======================================================================
 
+@require_POST
+@csrf_protect
 def generic_copy(request, what, obj_name=None, obj_newname=None):
    """
    Copies an object.
    """
-   if not test_user_authenticated(request): return login(request, next="/cobbler_web/%s/copy/%s/%s" % (what,obj_name,obj_newname))
+   if not test_user_authenticated(request): return login(request, next="/cobbler_web/%s/copy/%s/%s" % (what,obj_name,obj_newname), expired=True)
    # FIXME: shares all but one line with rename, merge it.
    if obj_name == None:
       return error_page(request,"You must specify a %s to rename" % what)
@@ -453,11 +487,13 @@ def generic_copy(request, what, obj_name=None, obj_newname=None):
 
 # ======================================================================
 
+@require_POST
+@csrf_protect
 def generic_delete(request, what, obj_name=None):
    """
    Deletes an object.
    """
-   if not test_user_authenticated(request): return login(request, next="/cobbler_web/%s/delete/%s" % (what,obj_name))
+   if not test_user_authenticated(request): return login(request, next="/cobbler_web/%s/delete/%s" % (what,obj_name), expired=True)
    # FIXME: consolidate code with above functions.
    if obj_name == None:
       return error_page(request,"You must specify a %s to delete" % what)
@@ -472,13 +508,15 @@ def generic_delete(request, what, obj_name=None):
 
 # ======================================================================
 
+@require_POST
+@csrf_protect
 def generic_domulti(request, what, multi_mode=None, multi_arg=None):
 
     """
     Process operations like profile reassignment, netboot toggling, and deletion
     which occur on all items that are checked on the list page.
     """
-    if not test_user_authenticated(request): return login(request, next="/cobbler_web/%s/multi/%s/%s" % (what,multi_mode,multi_arg))
+    if not test_user_authenticated(request): return login(request, next="/cobbler_web/%s/multi/%s/%s" % (what,multi_mode,multi_arg), expired=True)
 
     # FIXME: cleanup
     # FIXME: COMMENTS!!!11111???
@@ -531,7 +569,7 @@ def generic_domulti(request, what, multi_mode=None, multi_arg=None):
             return error_page(request,"You can only select one distro at a time to build an ISO for")
         options = { "standalone" : True, "distro": str(names[0]) }
         remote.background_buildiso(options, request.session['token'])
-    elif what == "profile" and multi_mode == "reposync":
+    elif what == "repo" and multi_mode == "reposync":
         options = { "repos" : names, "tries" : 3 }
         remote.background_reposync(options,request.session['token'])
     else:
@@ -543,10 +581,10 @@ def generic_domulti(request, what, multi_mode=None, multi_arg=None):
 # ======================================================================
 
 def import_prompt(request):
-   if not test_user_authenticated(request): return login(request, next="/cobbler_web/import/prompt")
+   if not test_user_authenticated(request): return login(request, next="/cobbler_web/import/prompt", expired=True)
    t = get_template('import.tmpl')
-   html = t.render(Context({
-       'version'  : remote.version(request.session['token']),
+   html = t.render(RequestContext(request,{
+       'version'  : remote.extended_version(request.session['token'])['version'],
        'username' : username,
    }))
    return HttpResponse(html)
@@ -557,11 +595,11 @@ def check(request):
    """
    Shows a page with the results of 'cobbler check'
    """
-   if not test_user_authenticated(request): return login(request, next="/cobbler_web/check")
+   if not test_user_authenticated(request): return login(request, next="/cobbler_web/check", expired=True)
    results = remote.check(request.session['token'])
    t = get_template('check.tmpl')
-   html = t.render(Context({
-       'version': remote.version(request.session['token']),
+   html = t.render(RequestContext(request,{
+       'version': remote.extended_version(request.session['token'])['version'],
        'username' : username,
        'results'  : results
    }))
@@ -569,15 +607,19 @@ def check(request):
 
 # ======================================================================
 
+@require_POST
+@csrf_protect
 def buildiso(request):
-    if not test_user_authenticated(request): return login(request, next="/cobbler_web/buildiso")
+    if not test_user_authenticated(request): return login(request, next="/cobbler_web/buildiso", expired=True)
     remote.background_buildiso({},request.session['token'])
     return HttpResponseRedirect('/cobbler_web/task_created')
 
 # ======================================================================
 
+@require_POST
+@csrf_protect
 def import_run(request):
-    if not test_user_authenticated(request): return login(request, next="/cobbler_web/import/prompt")
+    if not test_user_authenticated(request): return login(request, next="/cobbler_web/import/prompt", expired=True)
     options = {
         "name"  : request.POST.get("name",""),
         "path"  : request.POST.get("path",""),
@@ -593,7 +635,7 @@ def ksfile_list(request, page=None):
    """
    List all kickstart templates and link to their edit pages.
    """
-   if not test_user_authenticated(request): return login(request, next="/cobbler_web/ksfile/list")
+   if not test_user_authenticated(request): return login(request, next="/cobbler_web/ksfile/list", expired=True)
    ksfiles = remote.get_kickstart_templates(request.session['token'])
 
    ksfile_list = []
@@ -606,10 +648,10 @@ def ksfile_list(request, page=None):
          ksfile_list.append((ksfile,ksfile,None))
 
    t = get_template('ksfile_list.tmpl')
-   html = t.render(Context({
+   html = t.render(RequestContext(request,{
        'what':'ksfile',
        'ksfiles': ksfile_list,
-       'version': remote.version(request.session['token']),
+       'version': remote.extended_version(request.session['token'])['version'],
        'username': username,
        'item_count': len(ksfile_list[0]),
    }))
@@ -617,12 +659,12 @@ def ksfile_list(request, page=None):
 
 # ======================================================================
 
-
+@csrf_protect
 def ksfile_edit(request, ksfile_name=None, editmode='edit'):
    """
    This is the page where a kickstart file is edited.
    """
-   if not test_user_authenticated(request): return login(request, next="/cobbler_web/ksfile/edit/file:%s" % ksfile_name)
+   if not test_user_authenticated(request): return login(request, next="/cobbler_web/ksfile/edit/file:%s" % ksfile_name, expired=True)
    if editmode == 'edit':
       editable = False
    else:
@@ -635,24 +677,26 @@ def ksfile_edit(request, ksfile_name=None, editmode='edit'):
       ksdata = remote.read_or_write_kickstart_template(ksfile_name, True, "", request.session['token'])
 
    t = get_template('ksfile_edit.tmpl')
-   html = t.render(Context({
+   html = t.render(RequestContext(request,{
        'ksfile_name' : ksfile_name,
        'deleteable'  : deleteable,
        'ksdata'      : ksdata,
        'editable'    : editable,
        'editmode'    : editmode,
-       'version'     : remote.version(request.session['token']),
+       'version'     : remote.extended_version(request.session['token'])['version'],
        'username'    : username
    }))
    return HttpResponse(html)
 
 # ======================================================================
 
+@require_POST
+@csrf_protect
 def ksfile_save(request):
    """
    This page processes and saves edits to a kickstart file.
    """
-   if not test_user_authenticated(request): return login(request, next="/cobbler_web/ksfile/list")
+   if not test_user_authenticated(request): return login(request, next="/cobbler_web/ksfile/list", expired=True)
    # FIXME: error checking
 
    editmode = request.POST.get('editmode', 'edit')
@@ -680,7 +724,7 @@ def snippet_list(request, page=None):
    """
    This page lists all available snippets and has links to edit them.
    """
-   if not test_user_authenticated(request): return login(request, next="/cobbler_web/snippet/list")
+   if not test_user_authenticated(request): return login(request, next="/cobbler_web/snippet/list", expired=True)
    snippets = remote.get_snippets(request.session['token'])
 
    snippet_list = []
@@ -691,21 +735,22 @@ def snippet_list(request, page=None):
          snippet_list.append((snippet,snippet,None))
 
    t = get_template('snippet_list.tmpl')
-   html = t.render(Context({
+   html = t.render(RequestContext(request,{
        'what'     : 'snippet',
        'snippets' : snippet_list,
-       'version'  : remote.version(request.session['token']),
+       'version'  : remote.extended_version(request.session['token'])['version'],
        'username' : username
    }))
    return HttpResponse(html)
 
 # ======================================================================
 
+@csrf_protect
 def snippet_edit(request, snippet_name=None, editmode='edit'):
    """
    This page edits a specific snippet.
    """
-   if not test_user_authenticated(request): return login(request, next="/cobbler_web/edit/file:%s" % snippet_name)
+   if not test_user_authenticated(request): return login(request, next="/cobbler_web/edit/file:%s" % snippet_name, expired=True)
    if editmode == 'edit':
       editable = False
    else:
@@ -718,24 +763,26 @@ def snippet_edit(request, snippet_name=None, editmode='edit'):
       snippetdata = remote.read_or_write_snippet(snippet_name, True, "", request.session['token'])
 
    t = get_template('snippet_edit.tmpl')
-   html = t.render(Context({
+   html = t.render(RequestContext(request,{
        'snippet_name' : snippet_name,
        'deleteable'   : deleteable,
        'snippetdata'  : snippetdata,
        'editable'     : editable,
        'editmode'     : editmode,
-       'version'      : remote.version(request.session['token']),
+       'version'      : remote.extended_version(request.session['token'])['version'],
        'username'     : username
    }))
    return HttpResponse(html)
 
 # ======================================================================
 
+@require_POST
+@csrf_protect
 def snippet_save(request):
    """
    This snippet saves a snippet once edited.
    """
-   if not test_user_authenticated(request): return login(request, next="/cobbler_web/snippet/list")
+   if not test_user_authenticated(request): return login(request, next="/cobbler_web/snippet/list", expired=True)
    # FIXME: error checking
 
    editmode = request.POST.get('editmode', 'edit')
@@ -760,26 +807,86 @@ def snippet_save(request):
 
 # ======================================================================
 
-def settings(request):
-   """
-   This page presents a list of all the settings to the user.  They are not editable.
-   """
-   if not test_user_authenticated(request): return login(request, next="/cobbler_web/settings")
-   settings = remote.get_settings()
-   skeys = settings.keys()
-   skeys.sort()
+def setting_list(request):
+    """
+    This page presents a list of all the settings to the user.  They are not editable.
+    """
+    if not test_user_authenticated(request): return login(request, next="/cobbler_web/setting/list", expired=True)
+    settings = remote.get_settings()
+    skeys = settings.keys()
+    skeys.sort()
 
-   results = []
-   for k in skeys:
-      results.append([k,settings[k]])
+    results = []
+    for k in skeys:
+        results.append([k,settings[k]])
 
-   t = get_template('settings.tmpl')
-   html = t.render(Context({
-        'settings' : results,
-        'version'  : remote.version(request.session['token']),
-        'username' : username,
-   }))
-   return HttpResponse(html)
+    t = get_template('settings.tmpl')
+    html = t.render(RequestContext(request,{
+         'settings' : results,
+         'version'  : remote.extended_version(request.session['token'])['version'],
+         'username' : username,
+    }))
+    return HttpResponse(html)
+
+@csrf_protect
+def setting_edit(request, setting_name=None):
+    if not setting_name:
+        return HttpResponseRedirect('/cobbler_web/setting/list')
+    if not test_user_authenticated(request): return login(request, next="/cobbler_web/setting/edit/%s" % setting_name, expired=True)
+
+    settings = remote.get_settings()
+    if not settings.has_key(setting_name):
+        return error_page(request,"Unknown setting: %s" % setting_name)
+
+    cur_setting = {
+        'name'  : setting_name,
+        'value' : settings[setting_name],
+    }
+
+    fields = get_fields('setting', False, seed_item=cur_setting)
+    sections = {}
+    for field in fields:
+        bmo = field_info.BLOCK_MAPPINGS_ORDER[field['block_section']]
+        fkey = "%d_%s" % (bmo,field['block_section'])
+        if not sections.has_key(fkey):
+            sections[fkey] = {}
+            sections[fkey]['name'] = field['block_section']
+            sections[fkey]['fields'] = []
+        sections[fkey]['fields'].append(field)
+
+    t = get_template('generic_edit.tmpl')
+    html = t.render(RequestContext(request,{
+        'what'            : 'setting',
+        #'fields'          : fields,
+        'sections'        : sections,
+        'subobject'       : False,
+        'editmode'        : 'edit',
+        'editable'        : True,
+        'version'         : remote.version(request.session['token']),
+        'username'        : username,
+        'name'            : setting_name,
+    }))
+    return HttpResponse(html)
+
+@csrf_protect
+def setting_save(request):
+    if not test_user_authenticated(request): return login(request, next="/cobbler_web/setting/list", expired=True)
+
+    # load request fields and see if they are valid
+    setting_name = request.POST.get('name', "")
+    setting_value = request.POST.get('value', None)
+
+    if setting_name == "":
+        return error_page(request,"The setting name was not specified")
+
+    settings = remote.get_settings()
+    if not settings.has_key(setting_name):
+        return error_page(request,"Unknown setting: %s" % setting_name)
+
+    if remote.modify_setting(setting_name, setting_value, request.session['token']):
+        return error_page(request,"There was an error saving the setting")
+
+    return HttpResponseRedirect("/cobbler_web/setting/list")
 
 # ======================================================================
 
@@ -787,7 +894,7 @@ def events(request):
    """
    This page presents a list of all the events and links to the event log viewer.
    """
-   if not test_user_authenticated(request): return login(request, next="/cobbler_web/events")
+   if not test_user_authenticated(request): return login(request, next="/cobbler_web/events", expired=True)
    events = remote.get_events()
 
    events2 = []
@@ -800,9 +907,9 @@ def events(request):
    events2.sort(sorter)
 
    t = get_template('events.tmpl')
-   html = t.render(Context({
+   html = t.render(RequestContext(request,{
        'results'  : events2,
-       'version'  : remote.version(request.session['token']),
+       'version'  : remote.extended_version(request.session['token'])['version'],
        'username' : username
    }))
    return HttpResponse(html)
@@ -813,7 +920,7 @@ def eventlog(request, event=0):
    """
    Shows the log for a given event.
    """
-   if not test_user_authenticated(request): return login(request, next="/cobbler_web/eventlog/%s" % str(event))
+   if not test_user_authenticated(request): return login(request, next="/cobbler_web/eventlog/%s" % str(event), expired=True)
    event_info = remote.get_events()
    if not event_info.has_key(event):
       return HttpResponse("event not found")
@@ -831,10 +938,10 @@ def eventlog(request, event=0):
       'eventstate' : eventstate,
       'eventid'    : event,
       'eventtime'  : eventtime,
-      'version'    : remote.version(request.session['token']),
+      'version'    : remote.extended_version(request.session['token'])['version'],
       'username'  : username
    }
-   html = t.render(Context(vars))
+   html = t.render(RequestContext(request,vars))
    return HttpResponse(html)
 
 # ======================================================================
@@ -844,42 +951,50 @@ def random_mac(request, virttype="xenpv"):
    Used in an ajax call to fill in a field with a mac address.
    """
    # FIXME: not exposed in UI currently
-   if not test_user_authenticated(request): return login(request)
+   if not test_user_authenticated(request): return login(request, expired=True)
    random_mac = remote.get_random_mac(virttype, request.session['token'])
    return HttpResponse(random_mac)
 
 # ======================================================================
 
+@require_POST
+@csrf_protect
 def sync(request):
    """
    Runs 'cobbler sync' from the API when the user presses the sync button.
    """
-   if not test_user_authenticated(request): return login(request, next="/cobbler_web/sync")
+   if not test_user_authenticated(request): return login(request, next="/cobbler_web/sync", expired=True)
    remote.background_sync({"verbose":"True"},request.session['token'])
    return HttpResponseRedirect("/cobbler_web/task_created")
 
 # ======================================================================
 
+@require_POST
+@csrf_protect
 def reposync(request):
    """
    Syncs all repos that are configured to be synced.
    """
-   if not test_user_authenticated(request): return login(request, next="/cobbler_web/reposync")
+   if not test_user_authenticated(request): return login(request, next="/cobbler_web/reposync", expired=True)
    remote.background_reposync({ "names":"", "tries" : 3},request.session['token'])
    return HttpResponseRedirect("/cobbler_web/task_created")
 
 # ======================================================================
 
+@require_POST
+@csrf_protect
 def hardlink(request):
    """
    Hardlinks files between repos and install trees to save space.
    """
-   if not test_user_authenticated(request): return login(request, next="/cobbler_web/hardlink")
+   if not test_user_authenticated(request): return login(request, next="/cobbler_web/hardlink", expired=True)
    remote.background_hardlink({},request.session['token'])
    return HttpResponseRedirect("/cobbler_web/task_created")
 
 # ======================================================================
 
+@require_POST
+@csrf_protect
 def replicate(request):
    """
    Replicate configuration from the central cobbler server, configured
@@ -892,7 +1007,7 @@ def replicate(request):
    #settings = remote.get_settings()
    #options = settings # just load settings from file until we decide to ask user (later?)
    #remote.background_replicate(options, request.session['token'])
-   if not test_user_authenticated(request): return login(request, next="/cobbler_web/replicate")
+   if not test_user_authenticated(request): return login(request, next="/cobbler_web/replicate", expired=True)
    return HttpResponseRedirect("/cobbler_web/task_created")
 
 # ======================================================================
@@ -912,6 +1027,7 @@ def __names_from_dicts(loh,optional=True):
 
 # ======================================================================
 
+@csrf_protect
 def generic_edit(request, what=None, obj_name=None, editmode="new"):
 
    """
@@ -921,32 +1037,22 @@ def generic_edit(request, what=None, obj_name=None, editmode="new"):
    target = ""
    if obj_name != None:
        target = "/%s" % obj_name
-   if not test_user_authenticated(request): return login(request, next="/cobbler_web/%s/edit%s" % (what,target))
+   if not test_user_authenticated(request): return login(request, next="/cobbler_web/%s/edit%s" % (what,target), expired=True)
 
    obj = None
 
-   settings = remote.get_settings()
-
    child = False
    if what == "subprofile":
       what = "profile"
       child = True
-   
 
    if not obj_name is None:
       editable = remote.check_access_no_fail(request.session['token'], "modify_%s" % what, obj_name)
-      obj = remote.get_item(what, obj_name, True)
-   #
-   #   if obj.has_key('ctime'):
-   #      obj['ctime'] = time.ctime(obj['ctime'])
-   #   if obj.has_key('mtime'):
-   #      obj['mtime'] = time.ctime(obj['mtime'])
-
+      obj = remote.get_item(what, obj_name, False)
    else:
        editable = remote.check_access_no_fail(request.session['token'], "new_%s" % what, None)
        obj = None
 
-
    interfaces = {}
    if what == "system":
        if obj:
@@ -973,24 +1079,37 @@ def generic_edit(request, what=None, obj_name=None, editmode="new"):
 
    if what in ("distro","profile","system"):
        __tweak_field(fields, "mgmt_classes", "choices", __names_from_dicts(remote.get_mgmtclasses(),optional=False))
+       __tweak_field(fields, "os_version", "choices", remote.get_valid_os_versions())
+       __tweak_field(fields, "breed", "choices", remote.get_valid_breeds())
 
    # if editing save the fields in the session for comparison later
    if editmode == "edit":
        request.session['%s_%s' % (what,obj_name)] = fields
 
+   sections = {}
+   for field in fields:
+       bmo = field_info.BLOCK_MAPPINGS_ORDER[field['block_section']]
+       fkey = "%d_%s" % (bmo,field['block_section'])
+       if not sections.has_key(fkey):
+           sections[fkey] = {}
+           sections[fkey]['name'] = field['block_section']
+           sections[fkey]['fields'] = []
+       sections[fkey]['fields'].append(field)
+
    t = get_template('generic_edit.tmpl')
    inames = interfaces.keys()
    inames.sort()
-   html = t.render(Context({
+   html = t.render(RequestContext(request,{
        'what'            : what, 
-       'fields'          : fields, 
+       #'fields'          : fields, 
+       'sections'        : sections,
        'subobject'       : child,
        'editmode'        : editmode, 
        'editable'        : editable,
        'interfaces'      : interfaces,
        'interface_names' : inames,
        'interface_length': len(inames),
-       'version'         : remote.version(request.session['token']),
+       'version'         : remote.extended_version(request.session['token'])['version'],
        'username'        : username,
        'name'            : obj_name
    }))
@@ -999,12 +1118,14 @@ def generic_edit(request, what=None, obj_name=None, editmode="new"):
 
 # ======================================================================
 
+@require_POST
+@csrf_protect
 def generic_save(request,what):
 
     """
     Saves an object back using the cobbler API after clearing any 'generic_edit' page.
     """
-    if not test_user_authenticated(request): return login(request, next="/cobbler_web/%s/list" % what)
+    if not test_user_authenticated(request): return login(request, next="/cobbler_web/%s/list" % what, expired=True)
 
     # load request fields and see if they are valid
     editmode  = request.POST.get('editmode', 'edit')
@@ -1131,6 +1252,22 @@ def test_user_authenticated(request):
 
     remote = xmlrpclib.Server(url_cobbler_api, allow_none=True)
 
+    token = remote.login("", utils.get_shared_secret())
+    if ( (remote.get_authn_module_name(token) == 'authn_passthru' and 
+          request.META.has_key('REMOTE_USER')) and
+         ( (request.session.has_key('username') and
+            request.META['REMOTE_USER'] != request.session['username']) or
+           (not request.session.has_key('username')))):
+              try:
+                  username = request.META['REMOTE_USER'] 
+                  password = utils.get_shared_secret()
+                  token = remote.login(username, password)
+              except:
+                  token = None
+              if token:
+                  request.session['username'] = username
+                  request.session['token'] = token 
+  
     # if we have a token, get the associated username from
     # the remote server via XMLRPC. We then compare that to 
     # the value stored in the session.  If everything matches up,
@@ -1147,9 +1284,14 @@ def test_user_authenticated(request):
             pass
     return False
 
-def login(request, next=None):
-    return render_to_response('login.tmpl', {'next':next})
+@csrf_protect
+def login(request, next=None, message=None, expired=False):
+    if expired and not message:
+        message = "Sorry, either you need to login or your session expired."
+    return render_to_response('login.tmpl', RequestContext(request,{'next':next,'message':message}))
 
+@require_POST
+@csrf_protect
 def do_login(request):
     global remote
     global username
@@ -1177,8 +1319,10 @@ def do_login(request):
         else:
            return HttpResponseRedirect("/cobbler_web")
     else:
-        return login(request,nextsite)
+        return login(request,nextsite,message="Login failed, please try again")
 
+@require_POST
+@csrf_protect
 def do_logout(request):
     request.session['username'] = ""
     request.session['token'] = ""
